@@ -29,9 +29,6 @@
 
 #include <QCheckBox>
 #include <QLabel>
-#include <QPainter>
-#include <QRadioButton>
-#include <QSlider>
 #include <QTimer>
 
 
@@ -62,6 +59,8 @@
 #include "bgadvanced.h"
 #include "bgdialog.h"
 
+#include "kworkspace/screenpreviewwidget.h"
+
 #define NR_PREDEF_PATTERNS 6
 
 BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
@@ -150,9 +149,6 @@ BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
    // renderers
    if (m_numScreens > 1)
    {
-      m_renderer.resize(m_numScreens+2);
-      m_renderer.setAutoDelete(true);
-
       // Setup the merged-screen renderer
       KBackgroundRenderer * r = new KBackgroundRenderer(0, false, _config);
       m_renderer.insert( 0, r );
@@ -173,9 +169,6 @@ BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
    }
    else
    {
-      m_renderer.resize(1);
-      m_renderer.setAutoDelete(true);
-
       // set up the common desktop renderer
       KBackgroundRenderer * r = new KBackgroundRenderer(0, false, _config);
       m_renderer.insert(0, r);
@@ -210,6 +203,7 @@ BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
 BGDialog::~BGDialog()
 {
    delete m_pGlobals;
+   qDeleteAll(m_renderer);
 }
 
 KBackgroundRenderer * BGDialog::eRenderer()
@@ -257,7 +251,7 @@ void BGDialog::load()
    m_pGlobals->readSettings();
    getEScreen();
 
-   for (unsigned screen = 0; screen < m_renderer.size(); ++screen)
+   for (int screen = 0; screen < m_renderer.size(); ++screen)
    {
       unsigned eScreen = screen>1 ? screen-2 : 0;
       m_renderer[screen]->load( eScreen, (screen>0), true );
@@ -289,7 +283,7 @@ void BGDialog::save()
    // depending on which are the real settings
    // they both share Screen[0] in the config file
 
-      for (unsigned screen = 0; screen < m_renderer.size(); ++screen)
+      for (int screen = 0; screen < m_renderer.size(); ++screen)
       {
          if (screen == 1 && !m_pGlobals->commonScreenBackground())
             continue;
@@ -616,7 +610,7 @@ void BGDialog::slotWallpaperSelection()
       setWallpaper(dlg.selectedFile());
 
       int optionID = m_buttonGroupBackground->id(m_radioPicture);
-      m_buttonGroupBackground->setButton( optionID );
+      m_buttonGroupBackground->setSelected( optionID );
       slotWallpaperTypeChanged( optionID );
 
       emit changed(true);
@@ -653,7 +647,7 @@ void BGDialog::updateUI()
             m_comboWallpaperPos->setEnabled(false);
             m_lblWallpaperPos->setEnabled(false);
          }
-         m_buttonGroupBackground->setButton(
+         m_buttonGroupBackground->setSelected(
             m_buttonGroupBackground->id(m_radioNoPicture) );
       }
 
@@ -668,7 +662,7 @@ void BGDialog::updateUI()
             m_lblWallpaperPos->setEnabled(true);
          }
          setWallpaper(r->wallpaper());
-         m_buttonGroupBackground->setButton(
+         m_buttonGroupBackground->setSelected(
             m_buttonGroupBackground->id(m_radioPicture) );
       }
    }
@@ -683,7 +677,7 @@ void BGDialog::updateUI()
          m_comboWallpaperPos->setEnabled(true);
          m_lblWallpaperPos->setEnabled(true);
       }
-      m_buttonGroupBackground->setButton(
+      m_buttonGroupBackground->setSelected(
          m_buttonGroupBackground->id(m_radioSlideShow) );
    }
 
@@ -752,7 +746,7 @@ void BGDialog::updateUI()
        for (unsigned j = 0; j < m_numScreens; ++j)
        {
           m_renderer[j+2]->stop();
-          m_renderer[j+2]->setPreview( m_pMonitorArrangement->monitor(j)->size() );
+          m_renderer[j+2]->setPreview( m_pMonitorArrangement->monitor(j)->previewRect().size() );
           m_renderer[j+2]->start(true);
        }
     }
@@ -779,11 +773,11 @@ void BGDialog::slotPreviewDone(int screen_done)
    else if ( m_eScreen == 1 )
    {
       for (unsigned i = 0; i < m_pMonitorArrangement->numMonitors(); ++i)
-         m_pMonitorArrangement->monitor(i)->setPixmap(pm);
+         m_pMonitorArrangement->monitor(i)->setPreview(pm);
    }
    else
    {
-      m_pMonitorArrangement->monitor(screen_done)->setPixmap(pm);
+      m_pMonitorArrangement->monitor(screen_done)->setPreview(pm);
    }
 }
 
@@ -792,7 +786,7 @@ void BGDialog::slotImageDropped(const QString &uri)
    setWallpaper(uri);
 
    int optionID = m_buttonGroupBackground->id(m_radioPicture);
-   m_buttonGroupBackground->setButton( optionID );
+   m_buttonGroupBackground->setSelected( optionID );
    slotWallpaperTypeChanged( optionID );
 }
 
@@ -1044,7 +1038,7 @@ void BGDialog::slotSelectScreen(int screen)
       emit changed(true);
    else
    {
-      for (unsigned i = 0; i < m_renderer.size(); ++i)
+      for (int i = 0; i < m_renderer.size(); ++i)
       {
          if ( m_renderer[i]->isActive() )
             m_renderer[i]->stop();
@@ -1162,7 +1156,7 @@ void BGDialog::slotBlendReverse(bool b)
 
 void BGDialog::desktopResized()
 {
-   for (unsigned j = 0; j < m_renderer.size(); ++j )
+   for (int j = 0; j < m_renderer.size(); ++j )
    {
       KBackgroundRenderer * r = m_renderer[j];
       if( r->isActive())
