@@ -45,6 +45,7 @@ from the copyright holder.
 # include <rpc/rpc.h>
 # include <rpc/key_prot.h>
 extern int key_setnet( struct key_netstarg *arg );
+# include <X11/Xlib.h>
 #endif
 #ifdef K5AUTH
 # include <krb5/krb5.h>
@@ -135,7 +136,10 @@ displayStr( int lv, const char *msg )
 	gRecvInt();
 }
 
-#if !defined(USE_PAM) && (defined(HAVE_STRUCT_PASSWD_PW_EXPIRE) || defined(USESHADOW))
+#if (!defined(USE_PAM) && !defined(_AIX) \
+     && (defined(HAVE_STRUCT_PASSWD_PW_EXPIRE) || defined(USESHADOW) \
+         || (defined(KERBEROS) && defined(AFS)))) \
+    || defined(HAVE_CKCONNECTOR)
 static void
 displayMsg( int lv, const char *msg, ... )
 {
@@ -314,12 +318,12 @@ getPAMXauthData( const char *xauth_file )
 	struct pam_xauth_data *ret;
 
 	if (!(fp = fopen( xauth_file, "r" )))
-		return NULL;
+		return 0;
 
 	auth = XauReadAuth( fp );
 	fclose( fp );
 	if (!auth)
-		return NULL;
+		return 0;
 
 	if ((ret = malloc( sizeof(*ret) + auth->name_length + 1 + auth->data_length ))) {
 		ret->name = (char *)ret + sizeof(*ret);
@@ -376,12 +380,14 @@ doPAMAuth( const char *psrv, struct pam_data *pdata )
 # ifdef PAM_XDISPLAY
 	if ((pretc = pam_set_item( pamh, PAM_XDISPLAY,
 	                           displayName( td ) )) != PAM_SUCCESS)
-		goto pam_bail;
-	if (td->authFile && (pam_xauth = getPAMXauthData( td->authFile ))) {
+		debug( "setting PAM_XDISPLAY failed: %s\n",
+		       pam_strerror( 0, pretc ) );
+	else if (td->authFile && (pam_xauth = getPAMXauthData( td->authFile ))) {
 		pretc = pam_set_item( pamh, PAM_XAUTHDATA, pam_xauth );
 		free( pam_xauth );
 		if (pretc != PAM_SUCCESS)
-			goto pam_bail;
+			debug( "setting PAM_XAUTHDATA failed: %s\n",
+			       pam_strerror( 0, pretc ) );
 	}
 # endif
 	pam_set_item( pamh, PAM_USER_PROMPT, (void *)"Username:" );
@@ -603,11 +609,10 @@ verify( GConvFunc gconv, int rootok )
 		} else
 			if (!gconv( GCONV_PASS, 0 ))
 				V_RET_NP;
-		msg = NULL;
+		msg = 0;
 		if ((i = authenticate( curuser, curpass, &reenter, &msg ))) {
 			debug( "authenticate() failed: %s\n", msg );
-			if (msg)
-				free( msg );
+			free( msg );
 			loginfailed( curuser, hostname, tty );
 			if (i == ENOENT || i == ESAD)
 				V_RET_AUTH;
@@ -623,19 +628,17 @@ verify( GConvFunc gconv, int rootok )
 		if (!gconv( GCONV_USER, 0 ))
 			return False;
 		for (curret = 0;;) {
-			msg = NULL;
+			msg = 0;
 			if ((i = authenticate( curuser, curret, &reenter, &msg ))) {
 				debug( "authenticate() failed: %s\n", msg );
-				if (msg)
-					free( msg );
+				free( msg );
 				loginfailed( curuser, hostname, tty );
 				if (i == ENOENT || i == ESAD)
 					V_RET_AUTH;
 				else
 					V_RET_FAIL( 0 );
 			}
-			if (curret)
-				free( curret );
+			free( curret );
 			if (!reenter)
 				break;
 			if (!(curret = gconv( GCONV_HIDDEN, msg )))
@@ -737,7 +740,7 @@ verify( GConvFunc gconv, int rootok )
 # endif	 /* KERBEROS */
 
 # if defined(ultrix) || defined(__ultrix__)
-	if (authenticate_user( p, curpass, NULL ) < 0)
+	if (authenticate_user( p, curpass, 0 ) < 0)
 # elif defined(HAVE_PW_ENCRYPT)
 	if (strcmp( pw_encrypt( curpass, p->pw_passwd ), p->pw_passwd ))
 # elif defined(HAVE_CRYPT)
@@ -819,7 +822,7 @@ verify( GConvFunc gconv, int rootok )
 
 #elif defined(_AIX) /* USE_PAM */
 
-	msg = NULL;
+	msg = 0;
 	if (loginrestrictions( curuser,
 	                       ((td->displayType & d_location) == dForeign) ? S_RLOGIN : S_LOGIN,
 	                       tty, &msg ) == -1)
@@ -834,8 +837,7 @@ verify( GConvFunc gconv, int rootok )
 		gSendInt( V_AUTH );
 		V_RET;
 	}
-	if (msg)
-		free( (void *)msg );
+	free( msg );
 
 #endif /* USE_PAM || _AIX */
 
@@ -865,7 +867,7 @@ verify( GConvFunc gconv, int rootok )
 
 #   define DEFAULT_WARN	(2L * 7L)  /* Two weeks */
 
-		tim = time( NULL ) / 86400L;
+		tim = time( 0 ) / 86400L;
 
 #   ifdef HAVE_SETUSERCONTEXT
 		quietlog = login_getcapbool( lc, "hushlogin", False );
@@ -949,7 +951,7 @@ verify( GConvFunc gconv, int rootok )
 #  ifdef HAVE_SETUSERCONTEXT
 	     /* Do we ignore a nologin file? */
 	     !login_getcapbool( lc, "ignorenologin", False )) &&
-	    (!stat( (nolg = login_getcapstr( lc, "nologin", "", NULL )), &st ) ||
+	    (!stat( (nolg = login_getcapstr( lc, "nologin", "", 0 )), &st ) ||
 #  endif
 		 !stat( (nolg = _PATH_NOLOGIN), &st )))
 	{
@@ -975,7 +977,7 @@ verify( GConvFunc gconv, int rootok )
 
 /* restrict_time */
 #  if defined(HAVE_SETUSERCONTEXT) && defined(HAVE_AUTH_TIMEOK)
-	if (!auth_timeok( lc, time( NULL ) )) {
+	if (!auth_timeok( lc, time( 0 ) )) {
 		displayStr( V_MSG_ERR,
 		            "You are not allowed to login at the moment" );
 		gSendInt( V_FAIL );
@@ -1023,7 +1025,7 @@ static const char *envvars[] = {
 #ifdef _AIX
 	"AUTHSTATE", /* for kerberos */
 #endif
-	NULL
+	0
 };
 
 
@@ -1164,8 +1166,7 @@ mergeSessionArgs( int cansave )
 				}
 			iniSave( curdmrc, mfname );
 		}
-	if (mfname)
-		free( mfname );
+	free( mfname );
 }
 
 static int
@@ -1276,8 +1277,9 @@ startClient( volatile int *pid )
 	int i;
 
 	if (strCmp( dmrcuser, curuser )) {
-		if (curdmrc) { free( curdmrc ); curdmrc = 0; }
-		if (dmrcuser) { free( dmrcuser ); dmrcuser = 0; }
+		free( curdmrc );
+		free( dmrcuser );
+		curdmrc = dmrcuser = 0;
 	}
 
 #if defined(USE_PAM) || defined(_AIX)
@@ -1328,31 +1330,35 @@ startClient( volatile int *pid )
 			"remote-host-name", remoteHostName ?
 			                       (const char **)&remoteHostName : &spaceStr,
 # endif
-			NULL );
+			(char *)0 );
 # ifdef XDMCP
-	if (remoteHostName)
-		free( remoteHostName );
+	free( remoteHostName );
 # endif
 	debug( "ck status: %d\n", ckStatus );
 	if (!ckStatus) {
 		if (dbus_error_is_set( &error )) {
 			logError( "Cannot open ConsoleKit session: %s\n", error.message );
+			displayMsg( V_MSG_ERR,
+			            "Warning: Cannot open ConsoleKit session: %s",
+			             error.message );
 			dbus_error_free( &error );
 		} else {
 			logError( "Cannot open ConsoleKit session, likely OOM\n" );
+			displayStr( V_MSG_ERR,
+			            "Warning: Cannot open ConsoleKit session." );
 		}
 		ck_connector_unref( ckConnector );
-		V_RET;
+		ckConnector = 0;
 	}
 #endif
 
 #ifndef USE_PAM
 # ifdef _AIX
-	msg = NULL;
+	msg = 0;
 	loginsuccess( curuser, hostname, tty, &msg );
 	if (msg) {
 		debug( "loginsuccess() - %s\n", msg );
-		free( (void *)msg );
+		free( msg );
 	}
 # else /* _AIX */
 #  if defined(KERBEROS) && defined(AFS)
@@ -1363,7 +1369,7 @@ startClient( volatile int *pid )
 				logError( "setpag() for %s failed\n", curuser );
 				fail = True;
 			}
-			if ((ret = k_afsklog( NULL, NULL )) != KSUCCESS) {
+			if ((ret = k_afsklog( 0, 0 )) != KSUCCESS) {
 				logError( "AFS Warning: %s\n", krb_get_err_text( ret ) );
 				fail = True;
 			}
@@ -1625,7 +1631,7 @@ startClient( volatile int *pid )
 		 * Set the user's credentials: uid, gid, groups,
 		 * audit classes, user limits, and umask.
 		 */
-		if (setpcred( curuser, NULL ) == -1) {
+		if (setpcred( curuser, 0 ) == -1) {
 			logError( "setpcred for %s failed: %m\n", curuser );
 			goto cError;
 		}
@@ -1638,7 +1644,7 @@ startClient( volatile int *pid )
 		userEnviron = xCopyStrArr( 1, userEnviron );
 		userEnviron[0] = (char *)"USRENVIRON:";
 		if (setpenv( curuser, PENV_INIT | PENV_ARGV | PENV_NOEXEC,
-		             userEnviron, NULL ) != 0)
+		             userEnviron, 0 ) != 0)
 		{
 			logError( "Cannot set %s's process environment\n", curuser );
 			goto cError;
