@@ -43,6 +43,7 @@
 #include "qs_statusbar.h"
 #include "qs_matchitem.h"
 #include "qs_matchview.h"
+#include "qs_querymatchitem.h"
 
 //Widget dimensions
 const int WIDTH = 390;
@@ -67,14 +68,14 @@ class QsMatchView::Private
         QsCompletionBox *m_compBox;
         QList<MatchItem*> m_items;
         QString m_searchTerm;
-        QString m_itemCountSuffix;
         QGraphicsRectItem *m_descRect;
         QGraphicsTextItem *m_descText;
         int m_currentItem;
-        bool m_hasFocus;
-        bool m_itemsRemoved;
-        bool m_listVisible;
-        bool m_selectionMade;
+        bool m_hasFocus : 1;
+        bool m_itemsRemoved : 1;
+        bool m_listVisible : 1;
+        bool m_selectionMade : 1;
+        bool m_itemCountSuffixItems : 1;
 };
 
 QsMatchView::QsMatchView(QWidget *parent)
@@ -119,7 +120,7 @@ QsMatchView::QsMatchView(QWidget *parent)
 
     d->m_titleLabel = new QLabel(this);
     d->m_itemCountLabel = new QLabel(this);
-    d->m_itemCountSuffix = i18n("items");
+    d->m_itemCountSuffixItems = true;
 
     d->m_arrowButton = new QToolButton(this);
     d->m_arrowButton->setFocusPolicy(Qt::NoFocus);
@@ -176,28 +177,38 @@ void QsMatchView::reset()
     setDescriptionText(i18n("Type to search."));
 }
 
+bool queryMatchCompare(MatchItem *a, MatchItem *b)
+{
+    QueryMatchItem *m1 = qobject_cast<QueryMatchItem*>(a);
+    QueryMatchItem *m2 = qobject_cast<QueryMatchItem*>(b);
+    if (m1 && m2) {
+        return m2->match() < m1->match();
+    }
+    return a < b;
+}
+
 void QsMatchView::setItems(const QList<MatchItem*> &items, bool popup, bool append)
 {
     int spacing = MatchItem::ITEM_SIZE/2;
 
     int pos = spacing;
 
+    d->m_currentItem = -1;
+
     if (!append) {
         clear(true);
         d->m_compBox->clear();
-
-        d->m_currentItem = -1;
         d->m_items = items;
     } else {
-        // FIXME: This completely disregards item ranking
-        // Maybe should we just sort then scroll to previously selected item
-        if (!d->m_items.isEmpty()) {
+        /*if (!d->m_items.isEmpty()) {
             pos += d->m_items.last()->pos().x();
-        }
+        }*/
         d->m_items << items;
     }
+    qStableSort(d->m_items.begin(), d->m_items.end(), queryMatchCompare);
+    d->m_compBox->clear();
 
-    foreach(MatchItem *item, items) {
+    foreach(MatchItem *item, d->m_items) {
         if (item) {
             item->setPos(pos, SMALL_ICON_PADDING);
             item->scale(0.5, 0.5);
@@ -247,16 +258,20 @@ void QsMatchView::setTitle(const QString &title)
 
 void QsMatchView::setItemCount(int count)
 {
-    //TODO: place a context to aid translation
-    d->m_itemCountLabel->setText(i18n("%1 %2", count, d->m_itemCountSuffix));
+    if (d->m_itemCountSuffixItems) {
+        d->m_itemCountLabel->setText(i18np("1 item", "%1 items", count));
+    } else {
+        d->m_itemCountLabel->setText(i18np("1 action", "%1 actions", count));
+    }
+
     if (count) {
         d->m_arrowButton->show();
     }
 }
 
-void QsMatchView::setItemCountSuffix(const QString &suffix)
+void QsMatchView::setCountingActions(bool actions)
 {
-    d->m_itemCountSuffix = suffix;
+    d->m_itemCountSuffixItems = !actions;
 }
 
 void QsMatchView::setDescriptionText(const QString &text)
@@ -628,7 +643,7 @@ void QsMatchView::keyPressEvent(QKeyEvent *e)
     }
 
     //Don't add control characters to the search term
-    foreach (QChar c, e->text()) {
+    foreach (const QChar &c, e->text()) {
         if (c.isPrint()) {
             if (d->m_stack->currentIndex() == 1) {
                 d->m_searchTerm = d->m_lineEdit->text() + c;
