@@ -42,6 +42,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "scene.h"
 #include "effects.h"
 #include "deleted.h"
+#include "paintredirector.h"
+#include "tabbox.h"
 
 #include <X11/extensions/shape.h>
 #include <QX11Info>
@@ -50,6 +52,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <X11/extensions/sync.h>
 #endif
 
+#ifdef HAVE_XRENDER
+#include <X11/extensions/Xrender.h>
+#endif
+
+
 // Put all externs before the namespace statement to allow the linker
 // to resolve them properly
 
@@ -90,6 +97,7 @@ Client::Client( Workspace* ws )
     , delayedMoveResizeTimer( NULL )
     , in_group( NULL )
     , window_group( None )
+    , client_group( NULL )
     , in_layer( UnknownLayer )
     , ping_timer( NULL )
     , process_killer( NULL )
@@ -108,12 +116,20 @@ Client::Client( Workspace* ws )
     , border_right( 0 )
     , border_top( 0 )
     , border_bottom( 0 )
+    , padding_left( 0 )
+    , padding_right( 0 )
+    , padding_top( 0 )
+    , padding_bottom( 0 )
     , sm_stacking_order( -1 )
     , demandAttentionKNotifyTimer( NULL )
+    , paintRedirector( 0 )
+    , electricMaximizing( false )
     { // TODO: Do all as initialization
 
     // Set the initial mapping state
     mapping_state = Withdrawn;
+    quick_tile_mode = QuickTileNone;
+    geom_pretile = QRect( 0, 0, 0, 0 );
     desk = 0; // No desktop yet
 
     mode = PositionCenter;
@@ -138,6 +154,7 @@ Client::Client( Workspace* ws )
     modal = false;
     noborder = false;
     app_noborder = false;
+    motif_noborder = false;
     urgency = false;
     ignore_focus_stealing = false;
     demands_attention = false;
@@ -156,6 +173,10 @@ Client::Client( Workspace* ws )
 
     cmap = None;
 
+    // TabBoxClient
+    m_tabBoxClient = new TabBox::TabBoxClientImpl();
+    m_tabBoxClient->setClient( this );
+
     geom = QRect( 0, 0, 100, 100 ); // So that decorations don't start with size being (0,0)
     client_size = QSize( 100, 100 );
 #if defined(HAVE_XSYNC) || defined(HAVE_XDAMAGE)
@@ -182,6 +203,7 @@ Client::~Client()
     assert( block_geometry_updates == 0 );
     assert( !check_active_modal );
     delete bridge;
+    delete m_tabBoxClient;
     }
 
 // Use destroyClient() or releaseWindow(), Client instances cannot be deleted directly
@@ -211,7 +233,7 @@ void Client::releaseWindow( bool on_shutdown )
     finishWindowRules();
     ++block_geometry_updates;
     if( isOnCurrentDesktop() && isShown( true ))
-        addWorkspaceRepaint( geometry());
+        addWorkspaceRepaint( visibleRect() );
     // Grab X during the release to make removing of properties, setting to withdrawn state
     // and repareting to root an atomic operation (http://lists.kde.org/?l=kde-devel&m=116448102901184&w=2)
     grabXServer();
@@ -223,6 +245,8 @@ void Client::releaseWindow( bool on_shutdown )
     XUnmapWindow( display(), frameId()); // Destroying decoration would cause ugly visual effect
     destroyDecoration();
     cleanGrouping();
+    if( clientGroup() )
+        clientGroup()->remove( this, QRect(), true );
     if( !on_shutdown )
         {
         workspace()->removeClient( this, Allowed );
@@ -279,12 +303,14 @@ void Client::destroyClient()
     finishWindowRules();
     ++block_geometry_updates;
     if( isOnCurrentDesktop() && isShown( true ))
-        addWorkspaceRepaint( geometry());
+        addWorkspaceRepaint( visibleRect() );
     setModal( false );
     hidden = true; // So that it's not considered visible anymore
     workspace()->clientHidden( this );
     destroyDecoration();
     cleanGrouping();
+    if( clientGroup() )
+        clientGroup()->remove( this, QRect(), true );
     workspace()->removeClient( this, Allowed );
     client = None; // invalidate
     XDestroyWindow( display(), wrapper );
@@ -318,13 +344,16 @@ void Client::updateDecoration( bool check_workspace_pos, bool force )
         XReparentWindow( display(), decoration->widget()->winId(), frameId(), 0, 0 );
         decoration->widget()->lower();
         decoration->borders( border_left, border_right, border_top, border_bottom );
-        int save_workarea_diff_x = workarea_diff_x;
-        int save_workarea_diff_y = workarea_diff_y;
+        padding_left = padding_right = padding_top = padding_bottom = 0;
+        if (KDecorationUnstable *deco2 = dynamic_cast<KDecorationUnstable*>(decoration))
+            deco2->padding( padding_left, padding_right, padding_top, padding_bottom );
+        XMoveWindow( display(), decoration->widget()->winId(), -padding_left, -padding_top );
         move( calculateGravitation( false ));
         plainResize( sizeForClientSize( clientSize()), ForceGeometrySet );
-        workarea_diff_x = save_workarea_diff_x;
-        workarea_diff_y = save_workarea_diff_y;
         do_show = true;
+        paintRedirector = new PaintRedirector( decoration->widget());
+        connect( paintRedirector, SIGNAL( paintPending()), SLOT( repaintDecorationPending()));
+        resizeDecorationPixmaps();
         if( compositing() )
             discardWindowPixmap();
         if( scene != NULL )
@@ -337,7 +366,7 @@ void Client::updateDecoration( bool check_workspace_pos, bool force )
     if( check_workspace_pos )
         checkWorkspacePosition();
     blockGeometryUpdates( false );
-    if( do_show )
+    if( !noBorder() )
         decoration->widget()->show();
     updateFrameExtents();
     }
@@ -352,12 +381,11 @@ void Client::destroyDecoration()
         QPoint grav = calculateGravitation( true );
         border_left = border_right = border_top = border_bottom = 0;
         setMask( QRegion()); // Reset shape mask
-        int save_workarea_diff_x = workarea_diff_x;
-        int save_workarea_diff_y = workarea_diff_y;
         plainResize( sizeForClientSize( clientSize()), ForceGeometrySet );
         move( grav );
-        workarea_diff_x = save_workarea_diff_x;
-        workarea_diff_y = save_workarea_diff_y;
+        delete paintRedirector;
+        paintRedirector = NULL;
+        decorationPixmapLeft = decorationPixmapRight = decorationPixmapTop = decorationPixmapBottom = QPixmap();
         if( compositing() )
             discardWindowPixmap();
         if( scene != NULL && !deleting ) 
@@ -371,7 +399,16 @@ bool Client::checkBorderSizes( bool also_resize )
     {
     if( decoration == NULL )
         return false;
-    int new_left, new_right, new_top, new_bottom;
+
+    int new_left = 0, new_right = 0, new_top = 0, new_bottom = 0;
+    if (KDecorationUnstable *deco2 = dynamic_cast<KDecorationUnstable*>(decoration))
+        deco2->padding( new_left, new_right, new_top, new_bottom );
+    if (padding_left != new_left || padding_top != new_top)
+        XMoveWindow( display(), decoration->widget()->winId(), -new_left, -new_top );
+    padding_left = new_left;
+    padding_right = new_right;
+    padding_top = new_top;
+    padding_bottom = new_bottom;
     decoration->borders( new_left, new_right, new_top, new_bottom );
     if( new_left == border_left && new_right == border_right &&
         new_top == border_top && new_bottom == border_bottom )
@@ -396,12 +433,188 @@ bool Client::checkBorderSizes( bool also_resize )
     return true;
     }
 
-void Client::repaintDecoration()
+void Client::triggerDecorationRepaint()
     {
     if( decoration != NULL )
         decoration->widget()->update();
     }
 
+void Client::layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect &bottom, Client::CoordinateMode mode) const
+    {
+    QRect r = decoration->widget()->rect();
+    if (mode == WindowRelative)
+        r.translate(-padding_left, -padding_top);
+
+    NETStrut strut = info->frameOverlap();
+
+    // Ignore the overlap strut when compositing is disabled
+    if (!compositing() || !Workspace::self()->decorationSupportsFrameOverlap())
+        strut.left = strut.top = strut.right = strut.bottom = 0;
+    else if (strut.left == -1 && strut.top == -1 && strut.right == -1 && strut.bottom == -1)
+        {
+        top = QRect(r.x(), r.y(), r.width(), r.height() / 3);
+        left = QRect(r.x(), r.y() + top.height(), width() / 2, r.height() / 3);
+        right = QRect(r.x() + left.width(), r.y() + top.height(), r.width() - left.width(), left.height());
+        bottom = QRect(r.x(), r.y() + top.height() + left.height(), r.width(), r.height() - left.height() - top.height());
+        return;
+        }
+
+    top = QRect(r.x(), r.y(), r.width(), padding_top + border_top + strut.top);
+    bottom = QRect(r.x(), r.y() + r.height() - padding_bottom - border_bottom - strut.bottom,
+                   r.width(), padding_bottom + border_bottom + strut.bottom);
+    left = QRect(r.x(), r.y() + top.height(),
+                 padding_left + border_left + strut.left, r.height() - top.height() - bottom.height());
+    right = QRect(r.x() + r.width() - padding_right - border_right - strut.right, r.y() + top.height(),
+                  padding_right + border_right + strut.right, r.height() - top.height() - bottom.height());
+    }
+
+QRegion Client::decorationPendingRegion() const
+    {
+    if (!paintRedirector)
+        return QRegion();
+    return paintRedirector->scheduledRepaintRegion().translated( x() - padding_left, y() - padding_top );
+    }
+
+void Client::repaintDecorationPending()
+    {
+    if (compositing()) 
+        {
+        // The scene will update the decoration pixmaps in the next painting pass
+        // if it has not been already repainted before
+        const QRegion r = paintRedirector->scheduledRepaintRegion();
+        if (!r.isEmpty())
+            Workspace::self()->addRepaint( r.translated( x() - padding_left, y() - padding_top ) );
+        }
+    else
+        ensureDecorationPixmapsPainted();
+    }
+
+bool Client::decorationPixmapRequiresRepaint()
+    {
+    if (!paintRedirector)
+        return false;
+    QRegion r = paintRedirector->pendingRegion();
+    return !r.isEmpty();
+    }
+
+void Client::ensureDecorationPixmapsPainted()
+    {
+    if (!paintRedirector)
+        return;
+
+    QRegion r = paintRedirector->pendingRegion();
+    if (r.isEmpty())
+        return;
+
+    QPixmap p = paintRedirector->performPendingPaint();
+
+    QRect lr, rr, tr, br;
+    layoutDecorationRects( lr, tr, rr, br, DecorationRelative );
+
+    repaintDecorationPixmap( decorationPixmapLeft, lr, p, r );
+    repaintDecorationPixmap( decorationPixmapRight, rr, p, r );
+    repaintDecorationPixmap( decorationPixmapTop, tr, p, r );
+    repaintDecorationPixmap( decorationPixmapBottom, br, p, r );
+
+    if (!compositing())
+        {
+        // Blit the pixmaps to the frame window
+        layoutDecorationRects( lr, tr, rr, br, WindowRelative );
+#ifdef HAVE_XRENDER
+        if (Extensions::renderAvailable())
+            {
+            XRenderPictFormat* format = XRenderFindVisualFormat( display(), visual());
+            XRenderPictureAttributes pa;
+            pa.subwindow_mode = IncludeInferiors;
+            Picture pic = XRenderCreatePicture( display(), frameId(), format, CPSubwindowMode, &pa );
+            XRenderComposite( display(), PictOpSrc, decorationPixmapLeft.x11PictureHandle(), None, pic,
+                              0, 0, 0, 0, lr.x(), lr.y(), lr.width(), lr.height() );
+            XRenderComposite( display(), PictOpSrc, decorationPixmapRight.x11PictureHandle(), None, pic,
+                              0, 0, 0, 0, rr.x(), rr.y(), rr.width(), rr.height() );
+            XRenderComposite( display(), PictOpSrc, decorationPixmapTop.x11PictureHandle(), None, pic,
+                              0, 0, 0, 0, tr.x(), tr.y(), tr.width(), tr.height() );
+            XRenderComposite( display(), PictOpSrc, decorationPixmapBottom.x11PictureHandle(), None, pic,
+                              0, 0, 0, 0, br.x(), br.y(), br.width(), br.height() );
+            XRenderFreePicture( display(), pic ); // TODO don't recreate pictures all the time?
+            }
+        else
+#endif
+            {
+            XGCValues values;
+            values.subwindow_mode = IncludeInferiors;
+            GC gc = XCreateGC( display(), rootWindow(), GCSubwindowMode, &values );
+            XCopyArea( display(), decorationPixmapLeft.handle(), frameId(), gc, 0, 0,
+                       lr.width(), lr.height(), lr.x(), lr.y() );
+            XCopyArea( display(), decorationPixmapRight.handle(), frameId(), gc, 0, 0,
+                       rr.width(), rr.height(), rr.x(), rr.y() );
+            XCopyArea( display(), decorationPixmapTop.handle(), frameId(), gc, 0, 0,
+                       tr.width(), tr.height(), tr.x(), tr.y() );
+            XCopyArea( display(), decorationPixmapBottom.handle(), frameId(), gc, 0, 0,
+                       br.width(), br.height(), br.x(), br.y() );
+            XFreeGC( display(), gc );
+            }
+        }
+    else
+        XSync( display(), false );
+    }
+
+void Client::repaintDecorationPixmap( QPixmap& pix, const QRect& r, const QPixmap& src, QRegion reg )
+    {
+    if( !r.isValid())
+        return;
+    QRect b = reg.boundingRect();
+    reg &= r;
+    if( reg.isEmpty())
+        return;
+    QPainter pt( &pix );
+    pt.translate( -r.topLeft() );
+    pt.setCompositionMode( QPainter::CompositionMode_Source );
+    pt.setClipRegion( reg );
+    pt.drawPixmap( b.topLeft(), src );
+    pt.end();
+    }
+
+void Client::resizeDecorationPixmaps()
+    {
+    QRect lr, rr, tr, br;
+    layoutDecorationRects( lr, tr, rr, br, DecorationRelative );
+
+    decorationPixmapTop = QPixmap( tr.size() );
+    decorationPixmapBottom = QPixmap( br.size() );
+    decorationPixmapLeft = QPixmap( lr.size() );
+    decorationPixmapRight = QPixmap( rr.size() );
+#ifdef HAVE_XRENDER
+    if ( Extensions::renderAvailable() ) {
+        // Make sure the pixmaps are created with alpha channels
+        decorationPixmapLeft.fill( Qt::transparent );
+        decorationPixmapRight.fill( Qt::transparent );
+        decorationPixmapTop.fill( Qt::transparent );
+        decorationPixmapBottom.fill( Qt::transparent );
+    }
+#endif
+    triggerDecorationRepaint();
+    }
+
+QRect Client::transparentRect() const
+    {
+    if (isShade())
+        return QRect();
+
+    NETStrut strut = info->frameOverlap();
+    // Ignore the strut when compositing is disabled or the decoration doesn't support it
+    if (!compositing() || !Workspace::self()->decorationSupportsFrameOverlap())
+        strut.left = strut.top = strut.right = strut.bottom = 0;
+    else if (strut.left == -1 && strut.top == -1 && strut.right == -1 && strut.bottom == -1)
+        return QRect();
+
+    const QRect r = QRect(clientPos(), clientSize())
+                        .adjusted(strut.left, strut.top, -strut.right, -strut.bottom);
+    if (r.isValid())
+        return r;
+
+    return QRect();
+    }
+
 void Client::detectNoBorder()
     {
     if( shape())
@@ -461,13 +674,18 @@ void Client::resizeDecoration( const QSize& s )
     {
     if( decoration == NULL )
         return;
-    QSize oldsize = decoration->widget()->size();
-    decoration->resize( s );
-    if( oldsize == s )
+    QSize newSize = s + QSize(padding_left + padding_right, padding_top + padding_bottom);
+    QSize oldSize = decoration->widget()->size();
+    decoration->resize( newSize );
+    if( oldSize == newSize )
         {
-        QResizeEvent e( s, oldsize );
+        QResizeEvent e( newSize, oldSize );
         QApplication::sendEvent( decoration->widget(), &e );
         }
+    else // oldSize != newSize
+        {
+        resizeDecorationPixmaps();
+        }
     }
 
 bool Client::noBorder() const
@@ -477,7 +695,7 @@ bool Client::noBorder() const
 
 bool Client::userCanSetNoBorder() const
     {
-    return !isFullScreen() && !isShade();
+    return !isFullScreen() && !isShade() && ( clientGroup() == NULL || !(clientGroup()->items().count() > 1));
     }
 
 void Client::setNoBorder( bool set )
@@ -513,7 +731,7 @@ void Client::updateShape()
     if( compositing())
         {
         addRepaintFull();
-        addWorkspaceRepaint( geometry()); // In case shape change removes part of this window
+        addWorkspaceRepaint( visibleRect() ); // In case shape change removes part of this window
         }
     if( scene != NULL )
         scene->windowGeometryShapeChanged( this );
@@ -555,9 +773,10 @@ void Client::updateInputShape()
 
 void Client::setMask( const QRegion& reg, int mode )
     {
-    if( _mask == reg )
+    QRegion r = reg.translated( -padding_left, -padding_right ) & QRect( 0, 0, width(), height() );
+    if( _mask == r )
         return;
-    _mask = reg;
+    _mask = r;
     Window shape_window = frameId();
     if( shape() )
         { // The same way of applying a shape without strange intermediate states like above
@@ -566,13 +785,13 @@ void Client::setMask( const QRegion& reg, int mode )
                 0, 0, 1, 1, 0, 0, 0 );
         shape_window = shape_helper_window;
         }
-    if( reg.isEmpty() )
+    if( _mask.isEmpty() )
         XShapeCombineMask( display(), shape_window, ShapeBounding, 0, 0, None, ShapeSet );
     else if( mode == X::Unsorted )
-        XShapeCombineRegion( display(), shape_window, ShapeBounding, 0, 0, reg.handle(), ShapeSet );
+        XShapeCombineRegion( display(), shape_window, ShapeBounding, 0, 0, _mask.handle(), ShapeSet );
     else
         {
-        QVector< QRect > rects = reg.rects();
+        QVector< QRect > rects = _mask.rects();
         XRectangle* xrects = new XRectangle[rects.count()];
         for( int i = 0; i < rects.count(); ++i )
             {
@@ -636,11 +855,15 @@ bool Client::isMinimizable() const
         if( !shown_mainwindow )
             return true;
         }
+#if 0
     // This is here because kicker's taskbar doesn't provide separate entries
     // for windows with an explicitly given parent
     // TODO: perhaps this should be redone
+    // Disabled for now, since at least modal dialogs should be minimizable
+    // (resulting in the mainwindow being minimized too).
     if( transientFor() != NULL )
         return false;
+#endif
     if( !wantsTabFocus() ) // SELI, TODO: - NET::Utility? why wantsTabFocus() - skiptaskbar? ?
         return false;
     return true;
@@ -665,6 +888,10 @@ void Client::minimize( bool avoid_animation )
     workspace()->updateFocusChains( this, Workspace::FocusChainMakeLast );
     if( effects && !avoid_animation ) // TODO: Shouldn't it tell effects at least about the change?
         static_cast<EffectsHandlerImpl*>(effects)->windowMinimized( effectWindow());
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 void Client::unminimize( bool avoid_animation )
@@ -680,6 +907,10 @@ void Client::unminimize( bool avoid_animation )
     updateWindowRules();
     if( effects && !avoid_animation )
         static_cast<EffectsHandlerImpl*>( effects )->windowUnminimized( effectWindow() );
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 QRect Client::iconGeometry() const
@@ -742,7 +973,7 @@ void Client::setShade( ShadeMode mode )
     // TODO: All this unmapping, resizing etc. feels too much duplicated from elsewhere
     if ( isShade()) 
         { // shade_mode == ShadeNormal
-        addWorkspaceRepaint( geometry() );
+        addWorkspaceRepaint( visibleRect() );
         // Shade
         shade_geometry_change = true;
         QSize s( sizeForClientSize( QSize( clientSize() )));
@@ -783,6 +1014,10 @@ void Client::setShade( ShadeMode mode )
     workspace()->updateMinimizedOfTransients( this );
     decoration->shadeChange();
     updateWindowRules();
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 void Client::shadeHover()
@@ -812,7 +1047,7 @@ void Client::updateVisibility()
     {
     if( deleting )
         return;
-    if( hidden )
+    if( hidden && ( clientGroup() == NULL || clientGroup()->visible() == this ))
         {
         info->setState( NET::Hidden, NET::Hidden );
         setSkipTaskbar( true, false ); // Also hide from taskbar
@@ -822,7 +1057,8 @@ void Client::updateVisibility()
             internalHide( Allowed );
         return;
         }
-    setSkipTaskbar( original_skip_taskbar, false ); // Reset from 'hidden'
+    if( clientGroup() == NULL || clientGroup()->visible() == this )
+        setSkipTaskbar( original_skip_taskbar, false ); // Reset from 'hidden'
     if( minimized )
         {
         info->setState( NET::Hidden, NET::Hidden );
@@ -900,7 +1136,7 @@ void Client::internalHide( allowed_t )
         unmap( Allowed );
     if( old == Kept )
         updateHiddenPreview();
-    addWorkspaceRepaint( geometry() );
+    addWorkspaceRepaint( visibleRect() );
     workspace()->clientHidden( this );
     workspace()->checkUnredirect();
     }
@@ -915,7 +1151,7 @@ void Client::internalKeep( allowed_t )
     if( old == Unmapped || old == Withdrawn )
         map( Allowed );
     updateHiddenPreview();
-    addWorkspaceRepaint( geometry() );
+    addWorkspaceRepaint( visibleRect() );
     workspace()->clientHidden( this );
     workspace()->checkUnredirect();
     }
@@ -1212,6 +1448,10 @@ void Client::setDesktop( int desktop )
     workspace()->updateFocusChains( this, Workspace::FocusChainMakeFirst );
     updateVisibility();
     updateWindowRules();
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 /**
@@ -1234,6 +1474,10 @@ void Client::setOnAllDesktops( bool b )
         setDesktop( NET::OnAllDesktops );
     else
         setDesktop( workspace()->currentDesktop());
+
+    // Update states of all other windows in this group
+    if( clientGroup() )
+        clientGroup()->updateStates( this );
     }
 
 /**
@@ -1384,7 +1628,11 @@ void Client::setCaption( const QString& _s, bool force )
             info->setVisibleIconName( ( cap_iconic + cap_suffix ).toUtf8() );
 
         if( isManaged() && decoration != NULL )
+            {
+            if (client_group)
+                client_group->updateItems();
             decoration->captionChange();
+            }
         }
     }
 
@@ -1422,6 +1670,43 @@ QString Client::caption( bool full ) const
     return full ? cap_normal + cap_suffix : cap_normal;
     }
 
+void Client::dontMoveResize()
+    {
+    buttonDown = false;
+    stopDelayedMoveResize();
+    if( moveResizeMode )
+        finishMoveResize( false );
+    }
+
+void Client::setClientShown( bool shown )
+    {
+    if( deleting )
+        return; // Don't change shown status if this client is being deleted
+    if( shown && hidden )
+        {
+        map( Allowed );
+        hidden = false;
+        //updateVisibility();
+        //updateAllowedActions();
+        if( options->inactiveTabsSkipTaskbar )
+            setSkipTaskbar( false, false );
+        takeFocus( Allowed );
+        autoRaise();
+        workspace()->updateFocusChains( this, Workspace::FocusChainMakeFirst );
+        }
+    if( !shown && !hidden )
+        {
+        unmap( Allowed );
+        hidden = true;
+        //updateVisibility();
+        //updateAllowedActions();
+        if( options->inactiveTabsSkipTaskbar )
+            setSkipTaskbar( true, false ); // TODO: Causes reshuffle of the taskbar
+        workspace()->updateFocusChains( this, Workspace::FocusChainMakeLast );
+        addWorkspaceRepaint( visibleRect() );
+        }
+    }
+
 void Client::getWMHints()
     {
     XWMHints* hints = XGetWMHints( display(), window() );
@@ -1444,12 +1729,18 @@ void Client::getWMHints()
 
 void Client::getMotifHints()
     {
-    bool mnoborder, mresize, mmove, mminimize, mmaximize, mclose;
-    Motif::readFlags( client, mnoborder, mresize, mmove, mminimize, mmaximize, mclose );
-    if( mnoborder )
+    bool mgot_noborder, mnoborder, mresize, mmove, mminimize, mmaximize, mclose;
+    Motif::readFlags( client, mgot_noborder, mnoborder, mresize, mmove, mminimize, mmaximize, mclose );
+    if( mgot_noborder )
         {
-        noborder = true;
-        app_noborder =  true;
+        motif_noborder = mnoborder;
+        // If we just got a hint telling us to hide decorations, we do so.
+        if ( motif_noborder )
+            noborder = true;
+        // If the Motif hint is now telling us to show decorations, we only do so if the app didn't
+        // instruct us to hide decorations in some other way, though.
+        else if ( !motif_noborder && !app_noborder )
+            noborder = false;
         }
     if( !hasNETSupport() )
         { // NETWM apps should set type and size constraints
@@ -1522,7 +1813,7 @@ void Client::getWindowProtocols()
 
     if( XGetWMProtocols( display(), window(), &p, &n ))
         {
-        for( i = 0; i < n; i++ )
+        for( i = 0; i < n; ++i )
             {
             if( p[i] == atoms->wm_delete_window )
                 Pdeletewindow = 1;
@@ -1574,6 +1865,9 @@ void Client::getSyncCounter()
                 &attrs );
             }
         }
+
+    if( ret == Success )
+        XFree( propRet );
 #endif
 }
 
