@@ -41,6 +41,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "kdecoration.h"
 #include "rules.h"
 #include "toplevel.h"
+#include "clientgroup.h"
 
 #ifdef HAVE_XSYNC
 #include <X11/extensions/sync.h>
@@ -54,10 +55,16 @@ class KStartupInfoData;
 
 namespace KWin
 {
+namespace TabBox {
+
+class TabBoxClientImpl;
+}
+
 
 class Workspace;
 class Client;
 class Bridge;
+class PaintRedirector;
 
 class Client
     : public Toplevel
@@ -103,6 +110,7 @@ class Client
         QSize maxSize() const;
         virtual QPoint clientPos() const; // Inside of geometry()
         virtual QSize clientSize() const;
+        virtual QRect visibleRect() const;
 
         bool windowEvent( XEvent* e );
         virtual bool eventFilter( QObject* o, QEvent* e );
@@ -201,7 +209,7 @@ class Client
 
         void updateDecoration( bool check_workspace_pos, bool force = false );
         bool checkBorderSizes( bool also_resize );
-        void repaintDecoration();
+        void triggerDecorationRepaint();
 
         void updateShape();
 
@@ -216,6 +224,19 @@ class Client
         void resizeWithChecks( int w, int h, ForceGeometry_t force = NormalGeometrySet );
         void resizeWithChecks( const QSize& s, ForceGeometry_t force = NormalGeometrySet );
         void keepInArea( QRect area, bool partial = false );
+        void setElectricBorderMode( ElectricMaximizingMode mode );
+        ElectricMaximizingMode electricBorderMode() const;
+        void setElectricBorderMaximizing( bool maximizing );
+        bool isElectricBorderMaximizing() const;
+        QRect electricBorderMaximizeGeometry();
+
+        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
+
+        /** Set the quick tile mode ("snap") of this window.
+         * This will also handle preserving and restoring of window geometry as necessary.
+         * @param mode The tile mode (left/right) to give this window.
+         */
+        void setQuickTileMode( QuickTileMode mode, bool keyboard = false );
 
         void growHorizontal();
         void shrinkHorizontal();
@@ -226,6 +247,7 @@ class Client
         KShortcut shortcut() const;
         void setShortcut( const QString& cut );
 
+        WindowOperation mouseButtonToWindowOperation( Qt::MouseButtons button );
         bool performMouseCommand( Options::MouseCommand, const QPoint& globalPos, bool handled = false );
 
         QRect adjustedClientArea( const QRect& desktop, const QRect& area ) const;
@@ -280,7 +302,27 @@ class Client
         void cancelShadeHoverTimer();
         void cancelAutoRaise();
         void checkActiveModal();
+        StrutRect strutRect( StrutArea area ) const;
+        StrutRects strutRects() const;
         bool hasStrut() const;
+
+        // Tabbing functions
+        ClientGroup* clientGroup() const; // Returns a pointer to client_group
+        void setClientGroup( ClientGroup* group );
+        /*
+        *   If shown is true the client is mapped and raised, if false
+        *   the client is unmapped and hidden, this function is called
+        *   when the tabbing group of the client switches its visible
+        *   client.
+        */
+        void setClientShown( bool shown );
+        /*
+        *   When a click is done in the decoration and it calls the group
+        *   to change the visible client it starts to move-resize the new
+        *   client, this function stops it.
+        */
+        void dontMoveResize();
+
         /**
          * Whether or not the window has a strut that expands through the invisible area of
          * an xinerama setup where the monitors are not the same resolution.
@@ -297,10 +339,37 @@ class Client
             }
 
         // Decorations <-> Effects
-        QList<QRect> shadowQuads( ShadowType type ) const;
-        double shadowOpacity( ShadowType type ) const;
-        double shadowBrightness( ShadowType type ) const;
-        double shadowSaturation( ShadowType type ) const;
+        const QPixmap *topDecoPixmap() const { return &decorationPixmapTop; } 
+        const QPixmap *leftDecoPixmap() const { return &decorationPixmapLeft; } 
+        const QPixmap *bottomDecoPixmap() const { return &decorationPixmapBottom; } 
+        const QPixmap *rightDecoPixmap() const { return &decorationPixmapRight; } 
+
+        int paddingLeft() const { return padding_left; }
+        int paddingRight() const { return padding_right; }
+        int paddingTop() const { return padding_top; }
+        int paddingBottom() const { return padding_bottom; }
+
+        bool decorationPixmapRequiresRepaint();
+        void ensureDecorationPixmapsPainted();
+
+        QRect decorationRect() const {
+            return (decoration && decoration->widget()) ?
+                        decoration->widget()->rect().translated(-padding_left, -padding_top) :
+                        QRect(0, 0, width(), height());
+        }
+
+        QRect transparentRect() const;
+
+        QRegion decorationPendingRegion() const;
+
+        enum CoordinateMode {
+            DecorationRelative, // Relative to the top left corner of the decoration
+            WindowRelative      // Relative to the top left corner of the window
+        };
+        void layoutDecorationRects(QRect &left, QRect &top, QRect &right, QRect &bottom, CoordinateMode mode) const;
+        virtual void addRepaintFull();
+
+        TabBox::TabBoxClientImpl* tabBoxClient() const { return m_tabBoxClient; }
 
     private slots:
         void autoRaise();
@@ -343,8 +412,10 @@ class Client
         bool buttonPressEvent( Window w, int button, int state, int x, int y, int x_root, int y_root );
         bool buttonReleaseEvent( Window w, int button, int state, int x, int y, int x_root, int y_root );
         bool motionNotifyEvent( Window w, int state, int x, int y, int x_root, int y_root );
+        void checkQuickTilingMaximizationZones( int xroot, int yroot );
 
-        void processDecorationButtonPress( int button, int state, int x, int y, int x_root, int y_root );
+        bool processDecorationButtonPress( int button, int state, int x, int y, int x_root, int y_root,
+            bool ignoreMenu = false );
 
     protected:
         virtual void debug( kdbgstream& stream ) const;
@@ -356,12 +427,12 @@ class Client
         void demandAttentionKNotify();
         void syncTimeout();
         void delayedSetShortcut();
+        void repaintDecorationPending();
 
     private:
         void exportMappingState( int s ); // ICCCM 4.1.3.1, 4.1.4, NETWM 2.5.1
         bool isManaged() const; ///< Returns false if this client is not yet managed
         void updateAllowedActions( bool force = false );
-        QSize sizeForClientSize( const QSize&, Sizemode mode = SizemodeAny, bool noframe = false ) const;
         QRect fullscreenMonitorsArea( NETFullscreenMonitors topology ) const;
         void changeMaximize( bool horizontal, bool vertical, bool adjust );
         void checkMaximizeGeometry();
@@ -378,9 +449,7 @@ class Client
         void finishWindowRules();
         void setShortcutInternal( const KShortcut& cut );
 
-        void updateWorkareaDiffs();
         void checkDirection( int new_diff, int old_diff, QRect& rect, const QRect& area );
-        static int computeWorkareaDiff( int left, int right, int a_left, int a_right );
         void configureRequest( int value_mask, int rx, int ry, int rw, int rh, int gravity, bool from_tool );
         NETExtendedStrut strut() const;
         int checkShadeGeometry( int w, int h );
@@ -421,6 +490,8 @@ class Client
         void updateHiddenPreview();
 
         void updateInputShape();
+        void repaintDecorationPixmap( QPixmap& pix, const QRect& r, const QPixmap& src, QRegion reg );
+        void resizeDecorationPixmaps();
 
         Time readUserTimeMapTimestamp( const KStartupInfoId* asn_id, const KStartupInfoData* asn_data,
             bool session ) const;
@@ -454,6 +525,12 @@ class Client
             Kept ///< The frame should be unmapped, but is kept (For compositing)
             };
         MappingState mapping_state;
+
+        /** The quick tile mode of this window.
+         */
+        QuickTileMode quick_tile_mode;
+        QRect geom_pretile;
+
         void readTransient();
         Window verifyTransientFor( Window transient_for, bool set );
         void addTransient( Client* cl );
@@ -487,7 +564,8 @@ class Client
         uint hidden : 1; ///< Forcibly hidden by calling hide()
         uint modal : 1; ///< NET::Modal
         uint noborder : 1;
-        uint app_noborder : 1; ///< The app requested no border using something (window type, motif hints)
+        uint app_noborder : 1; ///< App requested no border via window type, shape extension, etc.
+        uint motif_noborder : 1; ///< App requested no border via Motif WM hints
         uint urgency : 1; ///< XWMHints, UrgencyHint
         uint ignore_focus_stealing : 1; ///< Don't apply focus stealing prevention to this client
         uint demands_attention : 1;
@@ -510,7 +588,6 @@ class Client
         QRect geom_restore;
         QRect geom_fs_restore;
         MaximizeMode maxmode_restore;
-        int workarea_diff_x, workarea_diff_y;
         QTimer* autoRaiseTimer;
         QTimer* shadeHoverTimer;
         QTimer* delayedMoveResizeTimer;
@@ -518,6 +595,7 @@ class Client
         QString cap_normal, cap_iconic, cap_suffix;
         Group* in_group;
         Window window_group;
+        ClientGroup* client_group;
         Layer in_layer;
         QTimer* ping_timer;
         QProcess* process_killer;
@@ -534,6 +612,7 @@ class Client
             };
         PendingGeometry_t pending_geometry_update;
         QRect geom_before_block;
+        QRect deco_rect_before_block;
         bool shade_geometry_change;
 #ifdef HAVE_XSYNC
         XSyncCounter sync_counter;
@@ -543,6 +622,7 @@ class Client
         QTimer* sync_timeout;
         bool sync_resize_pending;
         int border_left, border_right, border_top, border_bottom;
+        int padding_left, padding_right, padding_top, padding_bottom;
         QRegion _mask;
         static bool check_active_modal; ///< \see Client::checkActiveModal()
         KShortcut _shortcut;
@@ -552,6 +632,12 @@ class Client
         friend struct ResetupRulesProcedure;
         friend class GeometryUpdatesBlocker;
         QTimer* demandAttentionKNotifyTimer;
+        QPixmap decorationPixmapLeft, decorationPixmapRight, decorationPixmapTop, decorationPixmapBottom;
+        PaintRedirector* paintRedirector;
+        TabBox::TabBoxClientImpl* m_tabBoxClient;
+
+        bool electricMaximizing;
+        ElectricMaximizingMode electricMode;
 
         friend bool performTransiencyCheck();
     };
@@ -643,6 +729,16 @@ inline Group* Client::group()
     return in_group;
     }
 
+inline void Client::setClientGroup( ClientGroup* group )
+    {
+    client_group = group;
+    }
+
+inline ClientGroup* Client::clientGroup() const
+    {
+    return client_group;
+    }
+
 inline bool Client::isMinimized() const
     {
     return minimized;
@@ -655,7 +751,8 @@ inline bool Client::isActive() const
 
 inline bool Client::isShown( bool shaded_is_shown ) const
     {
-    return !isMinimized() && ( !isShade() || shaded_is_shown ) && !hidden;
+    return !isMinimized() && ( !isShade() || shaded_is_shown ) && !hidden &&
+        ( clientGroup() == NULL || clientGroup()->visible() == this );
     }
 
 inline bool Client::isHiddenInternal() const
@@ -763,6 +860,11 @@ inline QSize Client::clientSize() const
     return client_size;
     }
 
+inline QRect Client::visibleRect() const
+   {
+   return geometry().adjusted( -padding_left, -padding_top, padding_right, padding_bottom );
+   }
+
 inline void Client::setGeometry( const QRect& r, ForceGeometry_t force )
     {
     setGeometry( r.x(), r.y(), r.width(), r.height(), force );
@@ -820,34 +922,6 @@ inline bool Client::hiddenPreview() const
     return mapping_state == Kept;
     }
 
-inline QList<QRect> Client::shadowQuads( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowQuads( type );
-    return QList<QRect>();
-    }
-
-inline double Client::shadowOpacity( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowOpacity( type );
-    return 1.0;
-    }
-
-inline double Client::shadowBrightness( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowBrightness( type );
-    return 1.0;
-    }
-
-inline double Client::shadowSaturation( ShadowType type ) const
-    {
-    if( KDecorationUnstable* decoration2 = dynamic_cast< KDecorationUnstable* >( decoration ))
-        return decoration2->shadowSaturation( type );
-    return 1.0;
-    }
-
 KWIN_COMPARE_PREDICATE( WrapperIdMatchPredicate, Client, Window, cl->wrapperId() == value );
 
 } // namespace
