@@ -27,6 +27,9 @@
 
 */
 
+#include "kwindecoration.h"
+#include "preview.h"
+
 #include <assert.h>
 #include <QDir>
 #include <QFileInfo>
@@ -35,15 +38,12 @@
 #include <QGroupBox>
 #include <QGridLayout>
 #include <QCheckBox>
-#include <QTabWidget>
+#include <KComboBox>
+#include <KTabWidget>
 
 #include <QLabel>
 #include <QFile>
 #include <QSlider>
-//Added by qt3to4:
-#include <QPixmap>
-#include <QHBoxLayout>
-#include <QVBoxLayout>
 
 #include <QtDBus/QtDBus>
 
@@ -57,8 +57,6 @@
 #include <kdialog.h>
 #include <kaboutdata.h>
 
-#include "kwindecoration.h"
-#include "preview.h"
 #include <kdecoration_plugins_p.h>
 #include <kdecorationfactory.h>
 #include <kvbox.h>
@@ -75,28 +73,25 @@ K_EXPORT_PLUGIN(KWinDecoFactory("kcmkwindecoration"))
 KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &)
     : KCModule(KWinDecoFactory::componentData(), parent),
     kwinConfig(KSharedConfig::openConfig("kwinrc")),
-    pluginObject(0)
+    pluginObject(0),
+    library(NULL)
 {
     KConfigGroup style( kwinConfig, "Style");
     plugins = new KDecorationPreviewPlugins(kwinConfig);
 
 	QVBoxLayout* layout = new QVBoxLayout(this);
-    layout->setMargin(0);
-    layout->setSpacing(KDialog::spacingHint());
 
 // Save this for later...
 //	cbUseMiniWindows = new QCheckBox( i18n( "Render mini &titlebars for all windows"), checkGroup );
 //	QWhatsThis::add( cbUseMiniWindows, i18n( "Note that this option is not available on all styles yet" ) );
 
-	tabWidget = new QTabWidget( this );
+	tabWidget = new KTabWidget( this );
 	layout->addWidget( tabWidget );
 
 	// Page 1 (General Options)
 	QWidget *pluginPage = new QWidget( tabWidget );
 
 	QVBoxLayout* pluginLayout = new QVBoxLayout(pluginPage);
-	pluginLayout->setMargin(KDialog::marginHint());
-	pluginLayout->setSpacing(KDialog::spacingHint());
 
 	// decoration chooser
 	decorationList = new KComboBox( pluginPage );
@@ -106,10 +101,9 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 	pluginLayout->addWidget(decorationList);
 
 	pluginSettingsGrp = new QGroupBox( i18n("Decoration Options"), pluginPage );
-    QGridLayout *pluginSettingsLayout = new QGridLayout();
+    QVBoxLayout *pluginSettingsLayout = new QVBoxLayout();
 	pluginSettingsGrp->setFlat( true );
 	pluginSettingsLayout->setMargin( 0 );
-	pluginSettingsLayout->setSpacing( KDialog::spacingHint() );
 	pluginSettingsGrp->setLayout( pluginSettingsLayout );
 	pluginLayout->addWidget( pluginSettingsGrp );
 
@@ -117,25 +111,23 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 
 	// Border size chooser
 	lBorder = new QLabel (i18n("B&order size:"), pluginSettingsGrp);
-	cBorder = new QComboBox(pluginSettingsGrp);
+	cBorder = new KComboBox(pluginSettingsGrp);
 	lBorder->setBuddy(cBorder);
 	cBorder->setWhatsThis( i18n( "Use this combobox to change the border size of the decoration." ));
 	lBorder->hide();
 	cBorder->hide();
 	QHBoxLayout *borderSizeLayout = new QHBoxLayout();
-    pluginSettingsGrp->layout()->addItem( borderSizeLayout );
+    pluginSettingsLayout->addLayout( borderSizeLayout );
 	borderSizeLayout->addWidget(lBorder);
 	borderSizeLayout->addWidget(cBorder);
 	borderSizeLayout->addStretch();
 
 	pluginConfigWidget = new KVBox(pluginSettingsGrp);
-	pluginSettingsGrp->layout()->addWidget( pluginConfigWidget );
+	pluginSettingsLayout->addWidget( pluginConfigWidget );
 
 	// Page 2 (Button Selector)
 	QWidget* buttonPage = new QWidget( tabWidget );
 	QVBoxLayout* buttonLayout = new QVBoxLayout(buttonPage);
-	buttonLayout->setMargin(KDialog::marginHint());
-	buttonLayout->setSpacing(KDialog::spacingHint());
 
 	cbShowToolTips = new QCheckBox(
 			i18n("&Show window button tooltips"), buttonPage );
@@ -153,7 +145,8 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 	buttonLayout->addWidget( cbUseCustomButtonPositions );
 
 	// Add nifty dnd button modification widgets
-	buttonPositionWidget = new ButtonPositionWidget(buttonPage, "button_position_widget");
+	buttonPositionWidget = new ButtonPositionWidget(buttonPage);
+	buttonPositionWidget->setObjectName("button_position_widget");
 	buttonPositionWidget->setDecorationFactory(plugins->factory() );
 	QHBoxLayout* buttonControlLayout = new QHBoxLayout();
     buttonLayout->addLayout( buttonControlLayout );
@@ -163,9 +156,8 @@ KWinDecorationModule::KWinDecorationModule(QWidget* parent, const QVariantList &
 
 	// preview
 	QVBoxLayout* previewLayout = new QVBoxLayout();
-    previewLayout->setSpacing( KDialog::spacingHint() );
     layout->addLayout( previewLayout );
-	previewLayout->setMargin( KDialog::marginHint() );
+	previewLayout->setMargin( 0 );
 
 	preview = new KDecorationPreview( this );
 	previewLayout->addWidget(preview);
@@ -404,14 +396,11 @@ void KWinDecorationModule::resetPlugin( KConfigGroup& conf, const QString& curre
 	delete pluginObject;
 	pluginObject = 0;
 
-	// Use klibloader for library manipulation
-	KLibLoader* loader = KLibLoader::self();
-
 	// Free the old library if possible
-	if (!oldLibraryName.isNull())
-		loader->unloadLibrary( oldName );
+	if (library != NULL)
+		library->unload();
 
-	KLibrary* library = loader->library( currentName );
+	library = new KLibrary( currentName );
 	if (library != NULL)
 	{
                 KLibrary::void_function_ptr alloc_ptr = library->resolveFunction("allocate_config");
@@ -450,9 +439,15 @@ void KWinDecorationModule::readConfig( const KConfigGroup & conf )
 
 	oldLibraryName = currentLibraryName;
 	currentLibraryName = conf.readEntry("PluginLib",
-					((QPixmap::defaultDepth() > 8) ? "kwin3_ozone" : "kwin3_plastik"));
+					((QPixmap::defaultDepth() > 8) ? "kwin3_oxygen" : "kwin3_plastik"));
 	QString decoName = decorationName( currentLibraryName );
 
+	if (decoName.isEmpty())
+	{ // Selected decoration doesn't exist, use the default
+		currentLibraryName = ((QPixmap::defaultDepth() > 8) ? "kwin3_oxygen" : "kwin3_plastik");
+		decoName = decorationName( currentLibraryName );
+	}
+
 	// If we are using the "default" kde client, use the "default" entry.
 //	if (decoName.isEmpty())
 //		decoName = i18n("KDE 2");
@@ -555,8 +550,8 @@ void KWinDecorationModule::defaults()
 // Don't set default for now
 //	decorationList->setSelected(
 //		decorationList->findItem( i18n("KDE 2") ), true );  // KDE classic client
-	decorationList->setCurrentItem( i18n("Ozone"), true );  // KDE classic client
-	slotChangeDecoration(i18n("Ozone"));
+	decorationList->setCurrentItem( i18n("Oxygen"), true );  // KDE classic client
+	slotChangeDecoration(i18n("Oxygen"));
 
 	buttonPositionWidget->setButtonsLeft(KDecorationOptions::defaultTitleButtonsLeft());
 	buttonPositionWidget->setButtonsRight(KDecorationOptions::defaultTitleButtonsRight());
@@ -605,9 +600,9 @@ QString KWinDecorationModule::quickHelp() const
 		"as well as titlebar button positions and custom decoration options.</p>"
 		"To choose a theme for your window decoration click on its name and apply your choice by clicking the \"Apply\" button below."
 		" If you do not want to apply your choice you can click the \"Reset\" button to discard your changes."
-		"<p>You can configure each theme in the \"Configure [...]\" tab. There are different options specific for each theme.</p>"
-		"<p>In \"General Options (if available)\" you can activate the \"Buttons\" tab by checking the \"Use custom titlebar button positions\" box."
-		" In the \"Buttons\" tab you can change the positions of the buttons to your liking.</p>" );
+		"<p>You can configure each theme. There are different options specific for each theme.</p>"
+		"<p>On the \"Buttons\" tab check the \"Use custom titlebar button positions\" box "
+		"and you can change the positions of the buttons to your liking.</p>" );
 }
 
 #include "kwindecoration.moc"
