@@ -27,7 +27,7 @@
 #include <QCheckBox>
 #include <QRadioButton>
 #include <QLabel>
-#include <QComboBox>
+#include <KComboBox>
 #include <QHBoxLayout>
 #include <QFormLayout>
 #include <QtDBus/QtDBus>
@@ -57,21 +57,16 @@
 #define KWIN_DELAYFOCUS            "DelayFocus"
 #define KWIN_CLICKRAISE            "ClickRaise"
 #define KWIN_MOVE_RESIZE_MAXIMIZED "MoveResizeMaximizedWindows"
-#define KWIN_ALTTABMODE            "AltTabStyle"
-#define KWIN_TRAVERSE_ALL          "TraverseAll"
-#define KWIN_SHOW_POPUP            "ShowPopup"
-#define KWIN_ROLL_OVER_DESKTOPS    "RollOverDesktops"
 #define KWIN_SHADEHOVER            "ShadeHover"
 #define KWIN_SHADEHOVER_INTERVAL   "ShadeHoverInterval"
 #define KWIN_FOCUS_STEALING        "FocusStealingPreventionLevel"
 #define KWIN_HIDE_UTILITY          "HideUtilityWindowsForInactive"
+#define KWIN_INACTIVE_SKIP_TASKBAR "InactiveTabsSkipTaskbar"
+#define KWIN_AUTOGROUP_SIMILAR     "AutogroupSimilarWindows"
+#define KWIN_AUTOGROUP_FOREGROUND  "AutogroupInForeground"
 #define KWIN_SEPARATE_SCREEN_FOCUS "SeparateScreenFocus"
 #define KWIN_ACTIVE_MOUSE_SCREEN   "ActiveMouseScreen"
 
-// kwm config keywords
-#define KWM_ELECTRIC_BORDER                  "ElectricBorders"
-#define KWM_ELECTRIC_BORDER_DELAY            "ElectricBorderDelay"
-
 //CT 15mar 98 - magics
 #define KWM_BRDR_SNAP_ZONE                   "BorderSnapZone"
 #define KWM_BRDR_SNAP_ZONE_DEFAULT           10
@@ -121,10 +116,10 @@ KFocusConfig::KFocusConfig (bool _standAlone, KConfig *_config, const KComponent
 
     fcsBox->setLayout( gLay );
 
-    focusStealing = new QComboBox( this );
+    focusStealing = new KComboBox( this );
     focusStealing->addItem( i18nc( "Focus Stealing Prevention Level", "None" ));
     focusStealing->addItem( i18nc( "Focus Stealing Prevention Level", "Low" ));
-    focusStealing->addItem( i18nc( "Focus Stealing Prevention Level", "Normal" ));
+    focusStealing->addItem( i18nc( "Focus Stealing Prevention Level", "Medium" ));
     focusStealing->addItem( i18nc( "Focus Stealing Prevention Level", "High" ));
     focusStealing->addItem( i18nc( "Focus Stealing Prevention Level", "Extreme" ));
     wtstr = i18n( "<p>This option specifies how much KWin will try to prevent unwanted focus stealing "
@@ -136,8 +131,8 @@ KFocusConfig::KFocusConfig (bool _standAlone, KConfig *_config, const KComponent
                   "<li><em>Low:</em> Prevention is enabled; when some window does not have support "
                   "for the underlying mechanism and KWin cannot reliably decide whether to "
                   "activate the window or not, it will be activated. This setting may have both "
-                  "worse and better results than normal level, depending on the applications.</li>"
-                  "<li><em>Normal:</em> Prevention is enabled.</li>"
+                  "worse and better results than the medium level, depending on the applications.</li>"
+                  "<li><em>Medium:</em> Prevention is enabled.</li>"
                   "<li><em>High:</em> New windows get activated only if no window is currently active "
                   "or if they belong to the currently active application. This setting is probably "
                   "not really usable when not using mouse focus policy.</li>"
@@ -155,7 +150,7 @@ KFocusConfig::KFocusConfig (bool _standAlone, KConfig *_config, const KComponent
     gLay->addWidget(label, 0, 0, 1, 2);
     gLay->addWidget(focusStealing, 0, 2);
 
-    focusCombo =  new QComboBox(fcsBox);
+    focusCombo =  new KComboBox(fcsBox);
     focusCombo->setEditable( false );
     focusCombo->addItem(i18n("Click to Focus"), CLICK_TO_FOCUS);
     focusCombo->addItem(i18n("Focus Follows Mouse"), FOCUS_FOLLOWS_MOUSE);
@@ -266,56 +261,6 @@ KFocusConfig::KFocusConfig (bool _standAlone, KConfig *_config, const KComponent
 
     lay->addWidget(fcsBox);
 
-
-
-
-
-    kbdBox = new KButtonGroup(this);
-    kbdBox->setTitle(i18nc("@title:group", "Window Switching"));
-    QVBoxLayout *kLay = new QVBoxLayout(kbdBox);
-
-    altTabPopup = new QCheckBox( i18n("Show window list while switching windows"), kbdBox );
-    kLay->addWidget( altTabPopup );
-
-    wtstr = i18n("Hold down the Alt key and press the Tab key repeatedly to walk"
-                 " through the windows on the current desktop (the Alt+Tab"
-                 " combination can be reconfigured).\n\n"
-                 "If this checkbox is checked"
-                 " a popup widget is shown, displaying the icons of all windows to"
-                 " walk through and the title of the currently selected one.\n\n"
-                 "Otherwise, the focus is passed to a new window each time Tab"
-                 " is pressed, with no popup widget.  In addition, the previously"
-                 " activated window will be sent to the back in this mode.");
-    altTabPopup->setWhatsThis( wtstr );
-    connect(focusCombo, SIGNAL(activated(int)), this, SLOT(updateAltTabMode()));
-    kLay->addWidget(altTabPopup);
-
-    traverseAll = new QCheckBox( i18n( "&Traverse windows on all desktops" ), kbdBox );
-    kLay->addWidget( traverseAll );
-
-    wtstr = i18n( "Leave this option disabled if you want to limit walking through"
-                  " windows to the current desktop." );
-    traverseAll->setWhatsThis( wtstr );
-    kLay->addWidget(traverseAll);
-
-    rollOverDesktops = new QCheckBox( i18n("Desktop navi&gation wraps around"), kbdBox );
-    kLay->addWidget(rollOverDesktops);
-
-    wtstr = i18n( "Enable this option if you want keyboard or active desktop border navigation beyond"
-                  " the edge of a desktop to take you to the opposite edge of the new desktop." );
-    rollOverDesktops->setWhatsThis( wtstr );
-    kLay->addWidget(rollOverDesktops);
-
-    showPopupinfo = new QCheckBox( i18n("Popup desktop name on desktop &switch"), kbdBox );
-    kLay->addWidget(showPopupinfo);
-
-    wtstr = i18n( "Enable this option if you wish to see the current desktop"
-                  " name popup whenever the current desktop is changed." );
-    showPopupinfo->setWhatsThis( wtstr );
-    kLay->addWidget(showPopupinfo);
-
-    lay->addWidget(kbdBox);
-
     lay->addStretch();
 
     // Any changes goes to slotChanged()
@@ -327,10 +272,6 @@ KFocusConfig::KFocusConfig (bool _standAlone, KConfig *_config, const KComponent
     connect(delayFocus, SIGNAL(valueChanged(int)), SLOT(changed()));
     connect(separateScreenFocus, SIGNAL(clicked()), SLOT(changed()));
     connect(activeMouseScreen, SIGNAL(clicked()), SLOT(changed()));
-    connect(altTabPopup, SIGNAL(clicked()), SLOT(changed()));
-    connect(traverseAll, SIGNAL(clicked()), SLOT(changed()));
-    connect(rollOverDesktops, SIGNAL(clicked()), SLOT(changed()));
-    connect(showPopupinfo, SIGNAL(clicked()), SLOT(changed()));
 
     load();
 }
@@ -347,13 +288,6 @@ void KFocusConfig::setFocus(int foc)
 
     // this will disable/hide the auto raise delay widget if focus==click
     focusPolicyChanged();
-    updateAltTabMode();
-}
-
-void KFocusConfig::updateAltTabMode()
-{
-    // not KDE-style Alt+Tab with unreasonable focus policies
-    altTabPopup->setEnabled( focusCombo->currentIndex() == 0 || focusCombo->currentIndex() == 1 );
 }
 
 void KFocusConfig::setAutoRaiseInterval(int tb)
@@ -447,20 +381,13 @@ void KFocusConfig::updateActiveMouseScreen()
         setActiveMouseScreen( focusCombo->currentIndex() != 0 );
 }
 
-void KFocusConfig::setAltTabMode(bool a) {
-    altTabPopup->setChecked(a);
-}
-
-void KFocusConfig::setTraverseAll(bool a) {
-    traverseAll->setChecked(a);
-}
-
-void KFocusConfig::setRollOverDesktops(bool a) {
-    rollOverDesktops->setChecked(a);
-}
-
-void KFocusConfig::setShowPopupinfo(bool a) {
-    showPopupinfo->setChecked(a);
+void KFocusConfig::showEvent( QShowEvent *ev )
+{
+    if ( !standAlone ) {
+        QWidget::showEvent( ev );
+        return;
+    }
+    KCModule::showEvent( ev );
 }
 
 void KFocusConfig::load( void )
@@ -495,16 +422,6 @@ void KFocusConfig::load( void )
     // on by default for non click to focus policies
     setActiveMouseScreen( cg.readEntry(KWIN_ACTIVE_MOUSE_SCREEN, focusCombo->currentIndex() != 0 ));
 
-    key = cg.readEntry(KWIN_ALTTABMODE, "KDE");
-    setAltTabMode(key == "KDE");
-
-    setRollOverDesktops( cg.readEntry(KWIN_ROLL_OVER_DESKTOPS, true));
-
-    setShowPopupinfo( config->group("PopupInfo").readEntry(KWIN_SHOW_POPUP, false));
-
-    setTraverseAll( config->group("TabBox").readEntry(KWIN_TRAVERSE_ALL, false));
-
-
 //    setFocusStealing( cg.readEntry(KWIN_FOCUS_STEALING, 2 ));
     // TODO default to low for now
     setFocusStealing( cg.readEntry(KWIN_FOCUS_STEALING, 1 ));
@@ -545,17 +462,6 @@ void KFocusConfig::save( void )
     cg.writeEntry(KWIN_SEPARATE_SCREEN_FOCUS, separateScreenFocus->isChecked());
     cg.writeEntry(KWIN_ACTIVE_MOUSE_SCREEN, activeMouseScreen->isChecked());
 
-    if (altTabPopup->isChecked())
-        cg.writeEntry(KWIN_ALTTABMODE, "KDE");
-    else
-        cg.writeEntry(KWIN_ALTTABMODE, "CDE");
-
-    cg.writeEntry( KWIN_ROLL_OVER_DESKTOPS, rollOverDesktops->isChecked());
-
-    config->group("PopupInfo").writeEntry( KWIN_SHOW_POPUP, showPopupinfo->isChecked());
-
-    config->group("TabBox").writeEntry( KWIN_TRAVERSE_ALL , traverseAll->isChecked());
-
     cg.writeEntry(KWIN_FOCUS_STEALING, focusStealing->currentIndex());
 
 
@@ -586,10 +492,6 @@ void KFocusConfig::defaults()
 
     // on by default for non click to focus policies
     setActiveMouseScreen( focusCombo->currentIndex() != 0 );
-    setAltTabMode(true);
-    setTraverseAll( false );
-    setRollOverDesktops(true);
-    setShowPopupinfo(false);
     setDelayFocusEnabled();
     emit KCModule::changed(true);
 }
@@ -605,7 +507,7 @@ KAdvancedConfig::KAdvancedConfig (bool _standAlone, KConfig *_config, const KCom
 {
     QString wtstr;
     QLabel *label;
-    QGridLayout *lay = new QGridLayout (this);
+    QVBoxLayout *lay = new QVBoxLayout (this);
 
     //iTLabel = new QLabel(i18n("  Allowed overlap:\n"
     //                         "(% of desktop space)"),
@@ -647,53 +549,48 @@ KAdvancedConfig::KAdvancedConfig (bool _standAlone, KConfig *_config, const KCom
     kLay->addWidget(shadeHoverLabel, 1, 0);
     kLay->addWidget(shadeHover, 1, 1);
 
-    kLay->setRowStretch(2, 1);
-    lay->addWidget(shBox, 0, 0);
+    lay->addWidget(shBox);
+
+    //----------------
+    // Window tabbing
+
+    wtBox = new KButtonGroup(this);
+    wtBox->setTitle(i18n("Window Tabbing"));
+    QVBoxLayout *wtLay = new QVBoxLayout(wtBox);
+
+    inactiveTabsSkipTaskbar = new QCheckBox( i18n( "Hide inactive window tabs from the taskbar" ), this );
+    inactiveTabsSkipTaskbar->setVisible( false ); // TODO: We want translations in case this is fixed...
+    inactiveTabsSkipTaskbar->setWhatsThis(
+        i18n( "When turned on hide all tabs that are not active from the taskbar." ));
+    connect(inactiveTabsSkipTaskbar, SIGNAL(toggled(bool)), SLOT(changed()));
+    wtLay->addWidget( inactiveTabsSkipTaskbar );
+
+    autogroupSimilarWindows = new QCheckBox( i18n( "Automatically group similar windows" ), this );
+    autogroupSimilarWindows->setWhatsThis(
+        i18n( "When turned on attempt to automatically detect when a newly opened window is related"
+              " to an existing one and place them in the same window group." ));
+    connect(autogroupSimilarWindows, SIGNAL(toggled(bool)), SLOT(changed()));
+    wtLay->addWidget( autogroupSimilarWindows );
+
+    autogroupInForeground = new QCheckBox( i18n( "Switch to automatically grouped windows immediately" ), this );
+    autogroupInForeground->setWhatsThis(
+        i18n( "When turned on immediately switch to any new window tabs that were automatically added"
+              " to the current group." ));
+    connect(autogroupInForeground, SIGNAL(toggled(bool)), SLOT(changed()));
+    wtLay->addWidget( autogroupInForeground );
+
+    lay->addWidget(wtBox);
+
+    //----------------
 
     // Any changes goes to slotChanged()
     connect(shadeHoverOn, SIGNAL(toggled(bool)), SLOT(changed()));
     connect(shadeHover, SIGNAL(valueChanged(int)), SLOT(changed()));
 
-    electricBox = new KButtonGroup(this);
-    electricBox->setTitle(i18n("Active Desktop Borders"));
-    QGridLayout *bLay = new QGridLayout(electricBox);
-
-    electricBox->setWhatsThis( i18n("If this option is enabled, moving the mouse to a screen border"
-       " will change your desktop. This is e.g. useful if you want to drag windows from one desktop"
-       " to the other.") );
-    active_disable = new QRadioButton(i18n("D&isabled"), electricBox);
-    bLay->addWidget(active_disable, 0, 0, 1, 2);
-    active_move    = new QRadioButton(i18n("Only &when moving windows"), electricBox);
-    bLay->addWidget(active_move, 1, 0, 1, 2);
-    active_always  = new QRadioButton(i18n("A&lways enabled"), electricBox);
-    bLay->addWidget(active_always, 2, 0, 1, 2);
-
-    delays = new KIntNumInput(10, electricBox);
-    delays->setRange(0, MAX_EDGE_RES, 50);
-    delays->setSuffix(i18n(" ms"));
-    delays->setWhatsThis( i18n("Here you can set a delay for switching desktops using the active"
-       " borders feature. Desktops will be switched after the mouse has been pushed against a screen border"
-       " for the specified number of milliseconds.") );
-    delays->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
-    delaysLabel = new QLabel(i18n("Desktop &switch delay:"), this);
-    delaysLabel->setAlignment(Qt::AlignVCenter|Qt::AlignRight);
-    delaysLabel->setBuddy(delays);
-    bLay->addWidget(delaysLabel, 3, 0);
-    bLay->addWidget(delays, 3, 1);
-
-    connect( electricBox, SIGNAL(clicked(int)), this, SLOT(setEBorders()));
-
-    // Any changes goes to slotChanged()
-    connect(electricBox, SIGNAL(clicked(int)), SLOT(changed()));
-    connect(delays, SIGNAL(valueChanged(int)), SLOT(changed()));
-
-    lay->addWidget(electricBox, 0, 1);
-
-
     QGridLayout *vLay = new QGridLayout();
-    lay->addLayout( vLay, 1, 0, 1, 2 );
+    lay->addLayout( vLay );
 
-    placementCombo = new QComboBox(this);
+    placementCombo = new KComboBox(this);
     placementCombo->setEditable( false );
     placementCombo->addItem(i18n("Smart"), SMART_PLACEMENT);
     placementCombo->addItem(i18n("Maximizing"), MAXIMIZING_PLACEMENT);
@@ -739,8 +636,7 @@ KAdvancedConfig::KAdvancedConfig (bool _standAlone, KConfig *_config, const KCom
     connect(hideUtilityWindowsForInactive, SIGNAL(toggled(bool)), SLOT(changed()));
     vLay->addWidget( hideUtilityWindowsForInactive, 1, 0, 1, 2 );
 
-
-    lay->setRowStretch(2, 1);
+    lay->addStretch();
     load();
 
 }
@@ -766,6 +662,15 @@ void KAdvancedConfig::shadeHoverChanged(bool a) {
 }
 
 
+void KAdvancedConfig::showEvent( QShowEvent *ev )
+{
+    if ( !standAlone ) {
+        QWidget::showEvent( ev );
+        return;
+    }
+    KCModule::showEvent( ev );
+}
+
 void KAdvancedConfig::load( void )
 {
     KConfigGroup cg(config, "Windows");
@@ -773,9 +678,6 @@ void KAdvancedConfig::load( void )
     setShadeHover(cg.readEntry(KWIN_SHADEHOVER, false));
     setShadeHoverInterval(cg.readEntry(KWIN_SHADEHOVER_INTERVAL, 250));
 
-    setElectricBorders(cg.readEntry(KWM_ELECTRIC_BORDER, 0));
-    setElectricBorderDelay(cg.readEntry(KWM_ELECTRIC_BORDER_DELAY, 150));
-
     QString key;
     // placement policy --- CT 19jan98 ---
     key = cg.readEntry(KWIN_PLACEMENT);
@@ -813,6 +715,9 @@ void KAdvancedConfig::load( void )
 //  }
 
     setHideUtilityWindowsForInactive( cg.readEntry( KWIN_HIDE_UTILITY, true));
+    setInactiveTabsSkipTaskbar( cg.readEntry( KWIN_INACTIVE_SKIP_TASKBAR, false));
+    setAutogroupSimilarWindows( cg.readEntry( KWIN_AUTOGROUP_SIMILAR, false));
+    setAutogroupInForeground( cg.readEntry( KWIN_AUTOGROUP_FOREGROUND, true));
 
     emit KCModule::changed(false);
 }
@@ -828,9 +733,6 @@ void KAdvancedConfig::save( void )
     if (v<0) v = 0;
     cg.writeEntry(KWIN_SHADEHOVER_INTERVAL, v);
 
-    cg.writeEntry(KWM_ELECTRIC_BORDER, getElectricBorders());
-    cg.writeEntry(KWM_ELECTRIC_BORDER_DELAY,getElectricBorderDelay());
-
     // placement policy --- CT 31jan98 ---
     v =getPlacement();
     if (v == RANDOM_PLACEMENT)
@@ -854,6 +756,9 @@ void KAdvancedConfig::save( void )
         cg.writeEntry(KWIN_PLACEMENT, "Smart");
 
     cg.writeEntry(KWIN_HIDE_UTILITY, hideUtilityWindowsForInactive->isChecked());
+    cg.writeEntry(KWIN_INACTIVE_SKIP_TASKBAR, inactiveTabsSkipTaskbar->isChecked());
+    cg.writeEntry(KWIN_AUTOGROUP_SIMILAR, autogroupSimilarWindows->isChecked());
+    cg.writeEntry(KWIN_AUTOGROUP_FOREGROUND, autogroupInForeground->isChecked());
 
     if (standAlone)
     {
@@ -871,10 +776,11 @@ void KAdvancedConfig::defaults()
 {
     setShadeHover(false);
     setShadeHoverInterval(250);
-    setElectricBorders(0);
-    setElectricBorderDelay(150);
     setPlacement(SMART_PLACEMENT);
     setHideUtilityWindowsForInactive( true );
+    setInactiveTabsSkipTaskbar( false );
+    setAutogroupSimilarWindows( false );
+    setAutogroupInForeground( true );
     emit KCModule::changed(true);
 }
 
@@ -890,45 +796,22 @@ void KAdvancedConfig::setPlacement(int plac)
 }
 
 
-void KAdvancedConfig::setEBorders()
-{
-    delaysLabel->setEnabled(!active_disable->isChecked());
-    delays->setEnabled(!active_disable->isChecked());
-}
-
-int KAdvancedConfig::getElectricBorders()
-{
-    if (active_move->isChecked())
-       return 1;
-    if (active_always->isChecked())
-       return 2;
-    return 0;
-}
-
-int KAdvancedConfig::getElectricBorderDelay()
-{
-    return delays->value();
-}
-
-void KAdvancedConfig::setElectricBorders(int i){
-    switch(i)
-    {
-      case 1: active_move->setChecked(true); break;
-      case 2: active_always->setChecked(true); break;
-      default: active_disable->setChecked(true); break;
-    }
-    setEBorders();
-}
-
-void KAdvancedConfig::setElectricBorderDelay(int delay)
-{
-    delays->setValue(delay);
-}
-
 void KAdvancedConfig::setHideUtilityWindowsForInactive(bool s) {
     hideUtilityWindowsForInactive->setChecked( s );
 }
 
+void KAdvancedConfig::setInactiveTabsSkipTaskbar(bool s) {
+    inactiveTabsSkipTaskbar->setChecked( s );
+}
+
+void KAdvancedConfig::setAutogroupSimilarWindows(bool s) {
+    autogroupSimilarWindows->setChecked( s );
+}
+
+void KAdvancedConfig::setAutogroupInForeground(bool s) {
+    autogroupInForeground->setChecked( s );
+}
+
 KMovingConfig::~KMovingConfig ()
 {
     if (standAlone)
@@ -968,11 +851,6 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, const KCompone
                                         " to the top-left corner of the screen is displayed together with"
                                         " its size."));
 
-    QGridLayout *rLay = new QGridLayout();
-    bLay->addLayout(rLay);
-    rLay->setColumnStretch(0,0);
-    rLay->setColumnStretch(1,1);
-
     moveResizeMaximized = new QCheckBox( i18n("Allow moving and resizing o&f maximized windows"), windowsBox);
     bLay->addWidget(moveResizeMaximized);
     moveResizeMaximized->setWhatsThis( i18n("When enabled, this feature activates the border of maximized windows"
@@ -980,8 +858,6 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, const KCompone
                                               " just like for normal windows"));
 
 
-    bLay->addSpacing(10);
-
     lay->addWidget(windowsBox);
 
     //iTLabel = new QLabel(i18n("  Allowed overlap:\n"
@@ -1007,7 +883,7 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, const KCompone
     QGridLayout *kLay = new QGridLayout(MagicBox);
 
     BrdrSnap = new KIntNumInput(10, MagicBox);
-    BrdrSnap->setSpecialValueText( i18n("none") );
+    BrdrSnap->setSpecialValueText( i18nc("no border snap zone", "none") );
     BrdrSnap->setRange( 0, MAX_BRDR_SNAP);
     BrdrSnap->setSteps(1,10);
     BrdrSnap->setWhatsThis( i18n("Here you can set the snap zone for screen borders, i.e."
@@ -1021,7 +897,7 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, const KCompone
     kLay->addWidget(BrdrSnap, 0, 1);
 
     WndwSnap = new KIntNumInput(10, MagicBox);
-    WndwSnap->setSpecialValueText( i18n("none") );
+    WndwSnap->setSpecialValueText( i18nc("no window snap zone", "none") );
     WndwSnap->setRange( 0, MAX_WNDW_SNAP);
     WndwSnap->setSteps(1,10);
     WndwSnap->setWhatsThis( i18n("Here you can set the snap zone for windows, i.e."
@@ -1035,7 +911,7 @@ KMovingConfig::KMovingConfig (bool _standAlone, KConfig *_config, const KCompone
     kLay->addWidget(WndwSnap, 1, 1);
 
     CntrSnap = new KIntNumInput(10, MagicBox);
-    CntrSnap->setSpecialValueText( i18n("none") );
+    CntrSnap->setSpecialValueText( i18nc("no center snap zone", "none") );
     CntrSnap->setRange( 0, MAX_CNTR_SNAP);
     CntrSnap->setSteps(1,10);
     CntrSnap->setWhatsThis( i18n("Here you can set the snap zone for the screen center, i.e."
@@ -1124,6 +1000,15 @@ void KMovingConfig::slotCntrSnapChanged(int value) {
     CntrSnap->setSuffix(i18np(" pixel", " pixels", value));
 }
 
+void KMovingConfig::showEvent( QShowEvent *ev )
+{
+    if ( !standAlone ) {
+        QWidget::showEvent( ev );
+        return;
+    }
+    KCModule::showEvent( ev );
+}
+
 void KMovingConfig::load( void )
 {
     QString key;
