@@ -3,6 +3,7 @@
  This file is part of the KDE project.
 
 Copyright (C) 2006 Lubos Lunak <l.lunak@kde.org>
+Copyright (C) 2009 Fredrik Höglund <fredrik@kde.org>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -131,8 +132,12 @@ SceneXrender::SceneXrender( Workspace* ws )
         kError( 1212 ) << "XRender compositing setup failed";
         return;
         }
-    if( !selfCheck())
-        return;
+    if( !initting ) // see comment for opengl version
+        {
+        if( !selfCheck())
+            return;
+        selfCheckDone = true;
+        }
     init_ok = true;
     }
 
@@ -169,7 +174,24 @@ void SceneXrender::createBuffer()
 // Just like SceneOpenGL::selfCheck()
 bool SceneXrender::selfCheck()
     {
-    QImage img( 3, 2, QImage::Format_RGB32 );
+    QRegion reg = selfCheckRegion();
+    if( wspace->overlayWindow())
+        { // avoid covering the whole screen too soon
+        wspace->setOverlayShape( reg );
+        wspace->showOverlay();
+        }
+    selfCheckSetup();
+    flushBuffer( PAINT_SCREEN_REGION, reg );
+    bool ok = selfCheckFinish();
+    if( wspace->overlayWindow())
+        wspace->hideOverlay();
+    return ok;
+    }
+
+void SceneXrender::selfCheckSetup()
+    {
+    KXErrorHandler err;
+    QImage img( selfCheckWidth(), selfCheckHeight(), QImage::Format_RGB32 );
     img.setPixel( 0, 0, QColor( Qt::red ).rgb());
     img.setPixel( 1, 0, QColor( Qt::green ).rgb());
     img.setPixel( 2, 0, QColor( Qt::blue ).rgb());
@@ -177,44 +199,41 @@ bool SceneXrender::selfCheck()
     img.setPixel( 1, 1, QColor( Qt::black ).rgb());
     img.setPixel( 2, 1, QColor( Qt::white ).rgb());
     QPixmap pix = QPixmap::fromImage( img );
-    QList< QPoint > points = selfCheckPoints();
-    QRegion reg;
-    foreach( const QPoint& p, points )
-        reg |= QRect( p, pix.size());
-    if( wspace->overlayWindow())
-        { // avoid covering the whole screen too soon
-        wspace->setOverlayShape( reg );
-        wspace->showOverlay();
-        }
-    foreach( const QPoint& p, points )
+    foreach( const QPoint& p, selfCheckPoints())
         {
         XSetWindowAttributes wa;
         wa.override_redirect = True;
-        ::Window window = XCreateWindow( display(), rootWindow(), 0, 0, 3, 2, 0, QX11Info::appDepth(),
-            CopyFromParent, CopyFromParent, CWOverrideRedirect, &wa );
+        ::Window window = XCreateWindow( display(), rootWindow(), 0, 0, selfCheckWidth(), selfCheckHeight(),
+            0, QX11Info::appDepth(), CopyFromParent, CopyFromParent, CWOverrideRedirect, &wa );
         XSetWindowBackgroundPixmap( display(), window, pix.handle());
         XClearWindow( display(), window );
         XMapWindow( display(), window );
         // move the window one down to where the result will be rendered too, just in case
         // the render would fail completely and eventual check would try to read this window's contents
         XMoveWindow( display(), window, p.x() + 1, p.y());
-        XCompositeRedirectWindow( display(), window, CompositeRedirectManual );
+        XCompositeRedirectWindow( display(), window, CompositeRedirectAutomatic );
         Pixmap wpix = XCompositeNameWindowPixmap( display(), window );
         XWindowAttributes attrs;
         XGetWindowAttributes( display(), window, &attrs );
         XRenderPictFormat* format = XRenderFindVisualFormat( display(), attrs.visual );
         Picture pic = XRenderCreatePicture( display(), wpix, format, 0, 0 );
-        QRect rect( p.x(), p.y(), 3, 2 );
+        QRect rect( p.x(), p.y(), selfCheckWidth(), selfCheckHeight());
         XRenderComposite( display(), PictOpSrc, pic, None, buffer, 0, 0, 0, 0,
             rect.x(), rect.y(), rect.width(), rect.height());
+        XRenderFreePicture( display(), pic );
         XFreePixmap( display(), wpix );
         XDestroyWindow( display(), window );
         }
-    flushBuffer( PAINT_SCREEN_REGION, reg );
+    err.error( true ); // just sync and discard
+    }
+
+bool SceneXrender::selfCheckFinish()
+    {
+    KXErrorHandler err;
     bool ok = true;
-    foreach( const QPoint& p, points )
+    foreach( const QPoint& p, selfCheckPoints())
         {
-        QPixmap pix = QPixmap::grabWindow( rootWindow(), p.x(), p.y(), 3, 2 );
+        QPixmap pix = QPixmap::grabWindow( rootWindow(), p.x(), p.y(), selfCheckWidth(), selfCheckHeight());
         QImage img = pix.toImage();
 //        kDebug(1212) << "P:" << QColor( img.pixel( 0, 0 )).name();
 //        kDebug(1212) << "P:" << QColor( img.pixel( 1, 0 )).name();
@@ -234,8 +253,8 @@ bool SceneXrender::selfCheck()
             break;
             }
         }
-    if( wspace->overlayWindow())
-        wspace->hideOverlay();
+    if( err.error( true ))
+        ok = false;
     if( ok )
         kDebug( 1212 ) << "Compositing self-check passed.";
     if( !ok && options->disableCompositingChecks )
@@ -259,7 +278,18 @@ void SceneXrender::paint( QRegion damage, ToplevelList toplevels )
     paintScreen( &mask, &damage );
     if( wspace->overlayWindow()) // show the window only after the first pass, since
         wspace->showOverlay();   // that pass may take long
+    if( !selfCheckDone )
+        {
+        selfCheckSetup();
+        damage |= selfCheckRegion();
+        }
     flushBuffer( mask, damage );
+    if( !selfCheckDone )
+        {
+        if( !selfCheckFinish())
+            QTimer::singleShot( 0, Workspace::self(), SLOT( finishCompositing()));
+        selfCheckDone = true;
+        }
     // do cleanup
     stacking_order.clear();
     }
@@ -438,11 +468,14 @@ void SceneXrender::windowAdded( Toplevel* c )
 // SceneXrender::Window
 //****************************************
 
+QPixmap *SceneXrender::Window::temp_pixmap = 0;
+
 SceneXrender::Window::Window( Toplevel* c )
     : Scene::Window( c )
     , _picture( None )
     , format( XRenderFindVisualFormat( display(), c->visual()))
     , alpha( None )
+    , alpha_cached_opacity(0.0)
     {
     }
 
@@ -491,41 +524,114 @@ void SceneXrender::Window::discardAlpha()
 // Create XRender picture for the alpha mask.
 Picture SceneXrender::Window::alphaMask( double opacity )
     {
-    if( isOpaque() && opacity == 1.0 )
+    if( isOpaque() && qFuzzyCompare( opacity, 1.0 ) )
         return None;
-    if( alpha != None && alpha_cached_opacity != opacity )
+
+    bool created = false;
+    if( alpha == None )
         {
-        if( alpha != None )
-            XRenderFreePicture( display(), alpha );
-        alpha = None;
+        // Create a 1x1 8bpp pixmap containing the given opacity in the alpha channel.
+        Pixmap pixmap = XCreatePixmap( display(), rootWindow(), 1, 1, 8 );
+        XRenderPictFormat* format = XRenderFindStandardFormat( display(), PictStandardA8 );
+        XRenderPictureAttributes pa;
+        pa.repeat = True;
+        alpha = XRenderCreatePicture( display(), pixmap, format, CPRepeat, &pa );
+        XFreePixmap( display(), pixmap );
+        created = true;
         }
-    if( alpha != None )
-        return alpha;
-    if( opacity == 1.0 )
-        { // no need to create alpha mask
-        alpha_cached_opacity = 1.0;
-        return None;
+    if( created || !qFuzzyCompare( alpha_cached_opacity + 1.0, opacity + 1.0 ) )
+        {
+        XRenderColor col;
+        col.alpha = int( opacity * 0xffff );
+        XRenderFillRectangle( display(), PictOpSrc, alpha, &col, 0, 0, 1, 1 );
+        alpha_cached_opacity = opacity;
         }
-    // Create a 1x1 8bpp pixmap containing the given opacity in the alpha channel.
-    Pixmap pixmap = XCreatePixmap( display(), rootWindow(), 1, 1, 8 );
-    XRenderPictFormat* format = XRenderFindStandardFormat( display(), PictStandardA8 );
-    XRenderPictureAttributes pa;
-    pa.repeat = True;
-    alpha = XRenderCreatePicture( display(), pixmap, format, CPRepeat, &pa );
-    XFreePixmap( display(), pixmap );
-    XRenderColor col;
-    col.alpha = int( opacity * 0xffff );
-    alpha_cached_opacity = opacity;
-    XRenderFillRectangle( display(), PictOpSrc, alpha, &col, 0, 0, 1, 1 );
     return alpha;
     }
 
+// Maps window coordinates to screen coordinates
+QRect SceneXrender::Window::mapToScreen( int mask, const WindowPaintData &data, const QRect &rect ) const
+    {
+    QRect r = rect;
+
+    if( mask & PAINT_WINDOW_TRANSFORMED )
+        {
+        // Apply the window transformation
+        r.moveTo( r.x() * data.xScale + data.xTranslate,
+                  r.y() * data.yScale + data.yTranslate );
+        r.setWidth( r.width() * data.xScale );
+        r.setHeight( r.height() * data.yScale );
+        }
+
+    // Move the rectangle to the screen position
+    r.translate( x(), y() );
+
+    if( mask & PAINT_SCREEN_TRANSFORMED )
+        {
+        // Apply the screen transformation
+        r.moveTo( r.x() * screen_paint.xScale + screen_paint.xTranslate,
+                  r.y() * screen_paint.yScale + screen_paint.yTranslate );
+        r.setWidth( r.width() * screen_paint.xScale );
+        r.setHeight( r.height() * screen_paint.yScale );
+        }
+
+    return r;
+    }
+
+// Maps window coordinates to screen coordinates
+QPoint SceneXrender::Window::mapToScreen( int mask, const WindowPaintData &data, const QPoint &point ) const
+    {
+    QPoint pt = point;
+
+    if( mask & PAINT_WINDOW_TRANSFORMED )
+        {
+        // Apply the window transformation
+        pt.rx() = pt.x() * data.xScale + data.xTranslate;
+        pt.ry() = pt.y() * data.yScale + data.yTranslate;
+        }
+
+    // Move the point to the screen position
+    pt += QPoint(x(), y());
+
+    if( mask & PAINT_SCREEN_TRANSFORMED )
+        {
+        // Apply the screen transformation
+        pt.rx() = pt.x() * screen_paint.xScale + screen_paint.xTranslate;
+        pt.ry() = pt.y() * screen_paint.yScale + screen_paint.yTranslate;
+        }
+
+    return pt;
+    }
+
+void SceneXrender::Window::prepareTempPixmap(const QPixmap *left, const QPixmap *top,
+                                             const QPixmap *right, const QPixmap *bottom)
+    {
+    const QRect r = static_cast<Client*>( toplevel )->decorationRect();
+
+    if( !temp_pixmap )
+        temp_pixmap = new QPixmap( r.width(), r.height() );
+    else if( temp_pixmap->width() < r.width() || temp_pixmap->height() < r.height() )
+        *temp_pixmap = QPixmap( r.width(), r.height() );
+
+    temp_pixmap->fill( Qt::transparent );
+
+    Display *dpy = display();
+    XRenderComposite( dpy, PictOpSrc, top->x11PictureHandle(), None, temp_pixmap->x11PictureHandle(),
+                      0, 0, 0, 0, 0, 0, top->width(), top->height() );
+    XRenderComposite( dpy, PictOpSrc, left->x11PictureHandle(), None, temp_pixmap->x11PictureHandle(),
+                      0, 0, 0, 0, 0, top->height(), left->width(), left->height() );
+    XRenderComposite( dpy, PictOpSrc, right->x11PictureHandle(), None, temp_pixmap->x11PictureHandle(),
+                      0, 0, 0, 0, r.width() - right->width(), top->height(), right->width(), right->height() );
+    XRenderComposite( dpy, PictOpSrc, bottom->x11PictureHandle(), None, temp_pixmap->x11PictureHandle(),
+                      0, 0, 0, 0, 0, r.height() - bottom->height(), bottom->width(), bottom->height() );
+    }
+
 // paint the window
 void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintData data )
     {
     setTransformedShape( QRegion()); // maybe nothing will be painted
     // check if there is something to paint
-    bool opaque = isOpaque() && data.opacity == 1.0;
+    bool opaque = isOpaque() && qFuzzyCompare( data.opacity, 1.0 );
     /* HACK: It seems this causes painting glitches, disable temporarily
     if(( mask & PAINT_WINDOW_OPAQUE ) ^ ( mask & PAINT_WINDOW_TRANSLUCENT ))
         { // We are only painting either opaque OR translucent windows, not both
@@ -550,41 +656,72 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
     else
         filter = ImageFilterFast;
     // do required transformations
-    int x = toplevel->x();
-    int y = toplevel->y();
-    int width = toplevel->width();
-    int height = toplevel->height();
+    const QRect wr = mapToScreen(mask, data, QRect(0, 0, width(), height()));
+    const QRect cr = QRect(toplevel->clientPos(), toplevel->clientSize()); // Client rect (in the window)
+    const QRect dr = mapToScreen(mask, data, cr); // Destination rect
     double xscale = 1;
     double yscale = 1;
-    transformed_shape = shape();
+    bool scaled = false;
+
+    Client *client = dynamic_cast<Client*>( toplevel );
+    Deleted *deleted = dynamic_cast<Deleted*>( toplevel );
+    if ( client && Workspace::self()->decorationHasAlpha() )
+        transformed_shape = QRegion( client->decorationRect() );
+    else if( deleted && Workspace::self()->decorationHasAlpha() )
+        transformed_shape = QRegion( deleted->decorationRect() );
+    else
+        transformed_shape = shape();
+
+    XTransform xform = {{
+        { XDoubleToFixed( 1 ), XDoubleToFixed( 0 ), XDoubleToFixed( 0 ) },
+        { XDoubleToFixed( 0 ), XDoubleToFixed( 1),  XDoubleToFixed( 0 ) },
+        { XDoubleToFixed( 0 ), XDoubleToFixed( 0 ), XDoubleToFixed( 1 ) }
+    }};
+ 
+    XTransform identity = {{
+        { XDoubleToFixed( 1 ), XDoubleToFixed( 0 ), XDoubleToFixed( 0 ) },
+        { XDoubleToFixed( 0 ), XDoubleToFixed( 1),  XDoubleToFixed( 0 ) },
+        { XDoubleToFixed( 0 ), XDoubleToFixed( 0 ), XDoubleToFixed( 1 ) }
+    }};
+ 
+    XRenderPictureAttributes attr;
+
     if( mask & PAINT_WINDOW_TRANSFORMED )
         {
-        xscale *= data.xScale;
-        yscale *= data.yScale;
-        x += data.xTranslate;
-        y += data.yTranslate;
+        xscale = data.xScale;
+        yscale = data.yScale;
         }
     if( mask & PAINT_SCREEN_TRANSFORMED )
         {
         xscale *= screen_paint.xScale;
         yscale *= screen_paint.yScale;
-        x = int( x * screen_paint.xScale );
-        y = int( y * screen_paint.yScale );
-        x += screen_paint.xTranslate;
-        y += screen_paint.yTranslate;
         }
-    if( yscale != 1 || xscale != 1 )
+    if( !qFuzzyCompare( xscale, 1.0 ) || !qFuzzyCompare( yscale, 1.0 ) )
         {
-        XTransform xform = {{
-            { XDoubleToFixed( 1 / xscale ), XDoubleToFixed( 0 ), XDoubleToFixed( 0 ) },
-            { XDoubleToFixed( 0 ), XDoubleToFixed( 1 / yscale ), XDoubleToFixed( 0 ) },
-            { XDoubleToFixed( 0 ), XDoubleToFixed( 0 ), XDoubleToFixed( 1 ) }
-        }};
+        scaled = true;
+        xform.matrix[0][0] = XDoubleToFixed(1.0 / xscale);
+        xform.matrix[1][1] = XDoubleToFixed(1.0 / yscale);
+
         XRenderSetPictureTransform( display(), pic, &xform );
-        width = (int)(width * xscale);
-        height = (int)(height * yscale);
         if( filter == ImageFilterGood )
             XRenderSetPictureFilter( display(), pic, const_cast< char* >( "good" ), NULL, 0 );
+
+        // This is needed to avoid hitting a fallback in the radeon driver.
+        // The Render specification states that sampling pixels outside the
+        // source picture results in alpha=0 pixels. This can be achieved by
+        // setting the border color to transparent black, but since the border
+        // color has the same format as the texture, it only works when the
+        // texture has an alpha channel. So the driver falls back to software
+        // when the repeat mode is RepeatNone, the picture has a non-identity
+        // transformation matrix, and doesn't have an alpha channel.
+        // Since we only scale the picture, we can work around this by setting
+        // the repeat mode to RepeatPad.
+        if (!window()->hasAlpha())
+            {
+            attr.repeat = RepeatPad;
+            XRenderChangePicture( display(), pic, CPRepeat, &attr );
+            }
+
         // transform the shape for clipping in paintTransformedScreen()
         QVector< QRect > rects = transformed_shape.rects();
         for( int i = 0;
@@ -597,42 +734,112 @@ void SceneXrender::Window::performPaint( int mask, QRegion region, WindowPaintDa
             }
         transformed_shape.setRects( rects.constData(), rects.count());
         }
-    transformed_shape.translate( x, y );
+
+    transformed_shape.translate( mapToScreen( mask, data, QPoint(0, 0) ) );
     PaintClipper pcreg( region ); // clip by the region to paint
     PaintClipper pc( transformed_shape ); // clip by window's shape
     for( PaintClipper::Iterator iterator;
          !iterator.isDone();
          iterator.next())
         {
-        if( opaque )
+        if( client || deleted )
             {
-            XRenderComposite( display(), PictOpSrc, pic, None, buffer, 0, 0, 0, 0,
-                x, y, width, height);
-            // fake brightness change by overlaying black
-            XRenderColor col = { 0, 0, 0, 0xffff * ( 1 - data.brightness ) };
-            XRenderFillRectangle( display(), PictOpOver, buffer, &col, x, y, width, height );
+            bool noBorder = true;
+            const QPixmap *left = NULL;
+            const QPixmap *top = NULL;
+            const QPixmap *right = NULL;
+            const QPixmap *bottom = NULL;
+            QRect tr, lr, rr, br;
+            QRect decorationRect;
+            if( client && !client->noBorder() )
+                {
+                noBorder = client->noBorder();
+                client->ensureDecorationPixmapsPainted();
+
+                left   = client->leftDecoPixmap();
+                top    = client->topDecoPixmap();
+                right  = client->rightDecoPixmap();
+                bottom = client->bottomDecoPixmap();
+                client->layoutDecorationRects( lr, tr, rr, br, Client::WindowRelative );
+                decorationRect = client->decorationRect();
+                }
+            if( deleted && !deleted->noBorder() )
+                {
+                noBorder = deleted->noBorder();
+                left   = deleted->leftDecoPixmap();
+                top    = deleted->topDecoPixmap();
+                right  = deleted->rightDecoPixmap();
+                bottom = deleted->bottomDecoPixmap();
+                deleted->layoutDecorationRects( lr, tr, rr, br );
+                decorationRect = deleted->decorationRect();
+                }
+            if( !noBorder )
+                {
+                // Paint the decoration
+                Picture alpha = alphaMask( data.opacity * data.decoration_opacity );
+                Display *dpy = display();
+
+                if( !scaled )
+                    {
+                    tr = mapToScreen( mask, data, tr );
+                    lr = mapToScreen( mask, data, lr );
+                    rr = mapToScreen( mask, data, rr );
+                    br = mapToScreen( mask, data, br );
+
+                    XRenderComposite( dpy, PictOpOver, top->x11PictureHandle(), alpha, buffer,
+                                      0, 0, 0, 0, tr.x(), tr.y(), tr.width(), tr.height());
+                    XRenderComposite( dpy, PictOpOver, left->x11PictureHandle(), alpha, buffer,
+                                      0, 0, 0, 0, lr.x(), lr.y(), lr.width(), lr.height());
+                    XRenderComposite( dpy, PictOpOver, right->x11PictureHandle(), alpha, buffer,
+                                      0, 0, 0, 0, rr.x(), rr.y(), rr.width(), rr.height());
+                    XRenderComposite( dpy, PictOpOver, bottom->x11PictureHandle(), alpha, buffer,
+                                      0, 0, 0, 0, br.x(), br.y(), br.width(), br.height());
+                    }
+                else
+                    {
+                    const QRect r = mapToScreen( mask, data, decorationRect );
+                    prepareTempPixmap( left, top, right, bottom );
+                    XRenderSetPictureTransform( dpy, temp_pixmap->x11PictureHandle(), &xform );
+                    XRenderComposite( dpy, PictOpOver, temp_pixmap->x11PictureHandle(), alpha, buffer,
+                                      0, 0, 0, 0, r.x(), r.y(), r.width(), r.height() );
+                    XRenderSetPictureTransform( dpy, temp_pixmap->x11PictureHandle(), &identity );
+                    }
+                }
             }
-        else
+        if ( !(mask & PAINT_DECORATION_ONLY) )
+            {
+            // Paint the window contents
+            if( opaque )
+                {
+                XRenderComposite( display(), PictOpSrc, pic, None, buffer, cr.x() * xscale, cr.y() * yscale,
+                                  0, 0, dr.x(), dr.y(), dr.width(), dr.height());
+                }
+            else
+                {
+                Picture alpha = alphaMask( data.opacity );
+                XRenderComposite( display(), PictOpOver, pic, alpha, buffer, cr.x() * xscale, cr.y() * yscale,
+                                  0, 0, dr.x(), dr.y(), dr.width(), dr.height());
+                transformed_shape = QRegion();
+                }
+            }
+ 
+       if( data.brightness < 1.0 )
             {
-            Picture alpha = alphaMask( data.opacity );
-            XRenderComposite( display(), PictOpOver, pic, alpha, buffer, 0, 0, 0, 0,
-                x, y, width, height);
             // fake brightness change by overlaying black
             XRenderColor col = { 0, 0, 0, 0xffff * ( 1 - data.brightness ) * data.opacity };
-            XRenderFillRectangle( display(), PictOpOver, buffer, &col, x, y, width, height );
-            transformed_shape = QRegion();
+            XRenderFillRectangle( display(), PictOpOver, buffer, &col, wr.x(), wr.y(), wr.width(), wr.height() );
             }
         }
-    if( xscale != 1 || yscale != 1 )
+    if( scaled )
         {
-        XTransform xform = {{
-            { XDoubleToFixed( 1 ), XDoubleToFixed( 0 ), XDoubleToFixed( 0 ) },
-            { XDoubleToFixed( 0 ), XDoubleToFixed( 1 ), XDoubleToFixed( 0 ) },
-            { XDoubleToFixed( 0 ), XDoubleToFixed( 0 ), XDoubleToFixed( 1 ) }
-        }};
-        XRenderSetPictureTransform( display(), pic, &xform );
+        XRenderSetPictureTransform( display(), pic, &identity );
         if( filter == ImageFilterGood )
             XRenderSetPictureFilter( display(), pic, const_cast< char* >( "fast" ), NULL, 0 );
+        if (!window()->hasAlpha())
+            {
+            attr.repeat = RepeatNone;
+            XRenderChangePicture( display(), pic, CPRepeat, &attr );
+            }
         }
     }
 
