@@ -1,5 +1,5 @@
 /*  This file is part of the KDE project
-    
+
     Copyright (C) 2007 John Tapsell <tapsell@kde.org>
 
     This library is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 #include <QTextStream>
 
 //for sysconf
-#include <unistd.h>  
+#include <unistd.h>
 //for kill and setNice
 #include <sys/types.h>
 #include <signal.h>
@@ -83,13 +83,13 @@ extern int sys_ioprio_get(int, int);
 #define IOPRIO_WHO_PROCESS 1
 #define IOPRIO_CLASS_SHIFT 13
 
-/* Expose the kernel calls to usespace via syscall
+/* Expose the kernel calls to userspace via syscall
  * See man ioprio_set  and man ioprio_get   for information on these functions */
 static int ioprio_set(int which, int who, int ioprio)
 {
   return syscall(SYS_ioprio_set, which, who, ioprio);
 }
- 
+
 static int ioprio_get(int which, int who)
 {
   return syscall(SYS_ioprio_get, which, who);
@@ -107,30 +107,31 @@ namespace KSysGuard
     public:
       Private() { mProcDir = opendir( "/proc" );}
       ~Private();
-      inline bool readProcStatus(long pid, Process *process);
-      inline bool readProcStat(long pid, Process *process);
-      inline bool readProcStatm(long pid, Process *process);
-      inline bool readProcCmdline(long pid, Process *process);
+      inline bool readProcStatus(const QString &dir, Process *process);
+      inline bool readProcStat(const QString &dir, Process *process);
+      inline bool readProcStatm(const QString &dir, Process *process);
+      inline bool readProcCmdline(const QString &dir, Process *process);
       inline bool getNiceness(long pid, Process *process);
+      inline bool getIOStatistics(const QString &dir, Process *process);
       QFile mFile;
-      char mBuffer[PROCESS_BUFFER_SIZE+1]; //used as a buffer to read data into      
+      char mBuffer[PROCESS_BUFFER_SIZE+1]; //used as a buffer to read data into
       DIR* mProcDir;
   };
 
-ProcessesLocal::Private::~Private() 
-{ 
+ProcessesLocal::Private::~Private()
+{
     closedir(mProcDir);
 }
-  
+
 ProcessesLocal::ProcessesLocal() : d(new Private())
 {
 
 }
 
-bool ProcessesLocal::Private::readProcStatus(long pid, Process *process)
+bool ProcessesLocal::Private::readProcStatus(const QString &dir, Process *process)
 {
-    mFile.setFileName(QString("/proc/%1/status").arg(pid));
-    if(!mFile.open(QIODevice::ReadOnly | QIODevice::Text))
+    mFile.setFileName(dir + "status");
+    if(!mFile.open(QIODevice::ReadOnly))
         return false;      /* process has terminated in the meantime */
 
     process->uid = 0;
@@ -148,7 +149,7 @@ bool ProcessesLocal::Private::readProcStatus(long pid, Process *process)
 	        if(++found == 4) goto finish;
 	    }
 	    break;
-	  case 'U': 
+	  case 'U':
 	    if((unsigned int)size > sizeof("Uid:") && qstrncmp(mBuffer, "Uid:", sizeof("Uid:")-1) == 0) {
 		sscanf(mBuffer + sizeof("Uid:") -1, "%Ld %Ld %Ld %Ld", &process->uid, &process->euid, &process->suid, &process->fsuid );
 	        if(++found == 4) goto finish;
@@ -178,8 +179,8 @@ bool ProcessesLocal::Private::readProcStatus(long pid, Process *process)
 
 long ProcessesLocal::getParentPid(long pid) {
     Q_ASSERT(pid != 0);
-    d->mFile.setFileName(QString("/proc/%1/stat").arg(pid));
-    if(!d->mFile.open(QIODevice::ReadOnly | QIODevice::Text))
+    d->mFile.setFileName("/proc/" + QString::number(pid) + "/stat");
+    if(!d->mFile.open(QIODevice::ReadOnly))
         return 0;      /* process has terminated in the meantime */
 
     int size; //amount of data read in
@@ -204,116 +205,121 @@ long ProcessesLocal::getParentPid(long pid) {
     return atol(++word);
 }
 
-bool ProcessesLocal::Private::readProcStat(long pid, Process *ps)
+bool ProcessesLocal::Private::readProcStat(const QString &dir, Process *ps)
 {
-    QString filename = QString("/proc/%1/stat").arg(pid);
-    // As an optomization, if the last file read in was stat, then we already have this info in memory
-    if(mFile.fileName() != filename) {  
+    QString filename = dir + "stat";
+    // As an optimization, if the last file read in was stat, then we already have this info in memory
+    if(mFile.fileName() != filename) {
         mFile.setFileName(filename);
-        if(!mFile.open(QIODevice::ReadOnly | QIODevice::Text))
+        if(!mFile.open(QIODevice::ReadOnly))
             return false;      /* process has terminated in the meantime */
-	if( mFile.readLine( mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
-	    mFile.close();
-	    return false;
-	}
-	mFile.close();
+        if( mFile.readLine( mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
+            mFile.close();
+            return false;
+        }
+        mFile.close();
     }
 
     int current_word = 0;  //count from 0
     char *word = mBuffer;
     char status='\0';
-    long vmSize = 0;
-    long vmRSS = 0;
+    unsigned long long vmSize = 0;
+    unsigned long long vmRSS = 0;
     while(current_word < 23) {
-	    if(word[0] == ' ' ) {
-		    ++current_word;
-		    switch(current_word) {
-			    case 2: //status
-                              status=word[1];  // Look at the first letter of the status.  
-			                      // We analyze this after the while loop
-			      break;
-			    case 6: //ttyNo
-			      {
-			        int ttyNo = atoi(word+1);
-				int major = ttyNo >> 8;
-				int minor = ttyNo & 0xff;
-				switch(major) {
-				  case 136:
-				    ps->setTty(QByteArray("pts/") + QByteArray::number(minor));
-				    break;
-				  case 5:
-				    ps->setTty(QByteArray("tty"));
-				  case 4:
-				    if(minor < 64)
-				      ps->setTty(QByteArray("tty") + QByteArray::number(minor));
-				    else
-				      ps->setTty(QByteArray("ttyS") + QByteArray::number(minor-64));
-				    break;
-				  default:
-				    ps->setTty(QByteArray());
-				}
-			      }
-			      break;
-			    case 13: //userTime
-			      ps->setUserTime(atoll(word+1));
-			      break;
-			    case 14: //sysTime
-			      ps->setSysTime(atoll(word+1));
-			      break;
-			    case 18: //niceLevel
-			      ps->setNiceLevel(atoi(word+1));  /*Or should we use getPriority instead? */
-			      break;
-			    case 22: //vmSize
-			      vmSize = atol(word+1);
-			      break;
-			    case 23: //vmRSS
-			      vmRSS = atol(word+1);
-			      break;
-			    default:
-			      break;
-		    }
-	    } else if(word[0] == 0) {
-	    	return false; //end of data - serious problem
-	    }
-	    word++;
+        if(word[0] == ' ' ) {
+            ++current_word;
+            switch(current_word) {
+                case 2: //status
+                    status=word[1];  // Look at the first letter of the status.
+                    // We analyze this after the while loop
+                    break;
+                case 6: //ttyNo
+                    {
+                        int ttyNo = atoi(word+1);
+                        int major = ttyNo >> 8;
+                        int minor = ttyNo & 0xff;
+                        switch(major) {
+                            case 136:
+                                ps->setTty(QByteArray("pts/") + QByteArray::number(minor));
+                                break;
+                            case 5:
+                                ps->setTty(QByteArray("tty"));
+                            case 4:
+                                if(minor < 64)
+                                    ps->setTty(QByteArray("tty") + QByteArray::number(minor));
+                                else
+                                    ps->setTty(QByteArray("ttyS") + QByteArray::number(minor-64));
+                                break;
+                            default:
+                                ps->setTty(QByteArray());
+                        }
+                    }
+                    break;
+                case 13: //userTime
+                    ps->setUserTime(atoll(word+1));
+                    break;
+                case 14: //sysTime
+                    ps->setSysTime(atoll(word+1));
+                    break;
+                case 18: //niceLevel
+                    ps->setNiceLevel(atoi(word+1));  /*Or should we use getPriority instead? */
+                    break;
+                case 22: //vmSize
+                    vmSize = atoll(word+1);
+                    break;
+                case 23: //vmRSS
+                    vmRSS = atoll(word+1);
+                    break;
+                default:
+                    break;
+            }
+        } else if(word[0] == 0) {
+            return false; //end of data - serious problem
+        }
+        word++;
     }
 
     /* There was a "(ps->vmRss+3) * sysconf(_SC_PAGESIZE)" here in the original ksysguard code.  I have no idea why!  After comparing it to
      *   meminfo and other tools, this means we report the RSS by 12 bytes differently compared to them.  So I'm removing the +3
-     *   to be consistent.  NEXT TIME COMMENT STRANGE THINGS LIKE THAT! :-) */
-    ps->setVmRSS(vmRSS * sysconf(_SC_PAGESIZE) / 1024); /*convert to KiB*/
-    ps->setVmSize(vmSize /= 1024); /* convert to KiB */
+     *   to be consistent.  NEXT TIME COMMENT STRANGE THINGS LIKE THAT! :-)
+     *
+     *   Update: I think I now know why - the kernel allocates 3 pages for
+     *   tracking information about each the process. This memory isn't
+     *   included in vmRSS..*/
+    ps->setVmRSS(vmRSS * (sysconf(_SC_PAGESIZE) / 1024)); /*convert to KiB*/
+    ps->setVmSize(vmSize / 1024); /* convert to KiB */
 
     switch( status) {
-      case 'R':
-        ps->setStatus(Process::Running);
-	break;
-      case 'S':
-        ps->setStatus(Process::Sleeping);
-	break;
-      case 'D':
-        ps->setStatus(Process::DiskSleep);
-	break;
-      case 'Z':
-        ps->setStatus(Process::Zombie);
-	break;
-      case 'T':
-         ps->setStatus(Process::Stopped);
-         break;
-      case 'W':
-         ps->setStatus(Process::Paging);
-         break;
-      default:
-         ps->setStatus(Process::OtherStatus);
-         break;
+        case 'R':
+            ps->setStatus(Process::Running);
+            break;
+        case 'S':
+            ps->setStatus(Process::Sleeping);
+            break;
+        case 'D':
+            ps->setStatus(Process::DiskSleep);
+            break;
+        case 'Z':
+            ps->setStatus(Process::Zombie);
+            break;
+        case 'T':
+            ps->setStatus(Process::Stopped);
+            break;
+        case 'W':
+            ps->setStatus(Process::Paging);
+            break;
+        default:
+            ps->setStatus(Process::OtherStatus);
+            break;
     }
     return true;
 }
 
-bool ProcessesLocal::Private::readProcStatm(long pid, Process *process)
+bool ProcessesLocal::Private::readProcStatm(const QString &dir, Process *process)
 {
-    mFile.setFileName(QString("/proc/%1/statm").arg(pid));
-    if(!mFile.open(QIODevice::ReadOnly | QIODevice::Text))
+#ifdef _SC_PAGESIZE
+    mFile.setFileName(dir + "statm");
+    if(!mFile.open(QIODevice::ReadOnly))
         return false;      /* process has terminated in the meantime */
 
     if( mFile.readLine( mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
@@ -338,28 +344,30 @@ bool ProcessesLocal::Private::readProcStatm(long pid, Process *process)
 
     /* we use the rss - shared  to find the amount of memory just this app uses */
     process->vmURSS = process->vmRSS - (shared * sysconf(_SC_PAGESIZE) / 1024);
+#else
+    process->vmURSS = 0;
+#endif
     return true;
 }
 
 
-bool ProcessesLocal::Private::readProcCmdline(long pid, Process *process)
+bool ProcessesLocal::Private::readProcCmdline(const QString &dir, Process *process)
 {
-    if(!process->command.isNull()) return true; //only parse the cmdline once
-    mFile.setFileName(QString("/proc/%1/cmdline").arg(pid));
-    if(!mFile.open(QIODevice::ReadOnly | QIODevice::Text))
+    if(!process->command.isNull()) return true; //only parse the cmdline once.  This function takes up 25% of the CPU time :-/
+    mFile.setFileName(dir + "cmdline");
+    if(!mFile.open(QIODevice::ReadOnly))
         return false;      /* process has terminated in the meantime */
 
     QTextStream in(&mFile);
     process->command = in.readAll();
 
-    //cmdline seperates parameters with the NULL character
-    process->command.replace('\0', ' ');
-    process->command = process->command.trimmed();
+    //cmdline separates parameters with the NULL character
     if(!process->command.isEmpty()) {
-        QString processname = process->command;
-	processname.remove(QRegExp("^[^ ]*/"));
-	if(processname.startsWith(process->name))
-	    process->name = processname;
+        if(process->command.startsWith(process->name)) {
+            int index = process->command.indexOf(QChar('\0'));
+            process->name = process->command.left(index);
+        }
+        process->command.replace('\0', ' ');
     }
 
     mFile.close();
@@ -391,7 +399,7 @@ bool ProcessesLocal::Private::getNiceness(long pid, Process *process) {
     if(sched_getparam(pid, &param) == 0)
       process->niceLevel = param.sched_priority;
     else
-      process->niceLevel = 0;  //Error getting scheduler parameters. 
+      process->niceLevel = 0;  //Error getting scheduler parameters.
   }
 
 #ifdef HAVE_IONICE
@@ -409,15 +417,62 @@ bool ProcessesLocal::Private::getNiceness(long pid, Process *process) {
   return true;
 }
 
+bool ProcessesLocal::Private::getIOStatistics(const QString &dir, Process *process)
+{
+    QString filename = dir + "io";
+    // As an optimization, if the last file read in was io, then we already have this info in memory
+    mFile.setFileName(filename);
+    if(!mFile.open(QIODevice::ReadOnly))
+        return false;      /* process has terminated in the meantime */
+    if( mFile.read( mBuffer, sizeof(mBuffer)) <= 0) { //-1 indicates nothing read
+        mFile.close();
+        return false;
+    }
+    mFile.close();
+
+    int current_word = 0;  //count from 0
+    char *word = mBuffer;
+    while(current_word < 6 && word[0] != 0) {
+        if(word[0] == ' ' ) {
+            qlonglong number = atoll(word+1);
+            switch(current_word++) {
+                case 0: //rchar - characters read
+                    process->setIoCharactersRead(number);
+                    break;
+                case 1: //wchar - characters written
+                    process->setIoCharactersWritten(number);
+                    break;
+                case 2: //syscr - read syscall
+                    process->setIoReadSyscalls(number);
+                    break;
+                case 3: //syscw - write syscall
+                    process->setIoWriteSyscalls(number);
+                    break;
+                case 4: //read_bytes - bytes actually read from I/O
+                    process->setIoCharactersActuallyRead(number);
+                    break;
+                case 5: //write_bytes - bytes actually written to I/O
+                    process->setIoCharactersActuallyWritten(number);
+                default:
+                    break;
+            }
+        }
+        word++;
+    }
+    return true;
+}
 bool ProcessesLocal::updateProcessInfo( long pid, Process *process)
 {
-    if(!d->readProcStat(pid, process)) return false;
-    if(!d->readProcStatus(pid, process)) return false;
-    if(!d->readProcStatm(pid, process)) return false;
-    if(!d->readProcCmdline(pid, process)) return false;
-    if(!d->getNiceness(pid, process)) return false;
+    bool success = true;
+    QString dir = "/proc/" + QString::number(pid) + '/';
+    if(!d->readProcStat(dir, process)) success = false;
+    if(!d->readProcStatus(dir, process)) success = false;
+    if(!d->readProcStatm(dir, process)) success = false;
+    if(!d->readProcCmdline(dir, process)) success = false;
+    if(!d->getNiceness(pid, process)) success = false;
+    if(mUpdateFlags.testFlag(Processes::IOStatistics) && !d->getIOStatistics(dir, process)) success = false;
 
-    return true;
+    return success;
 }
 
 QSet<long> ProcessesLocal::getAllPids( )
@@ -495,14 +550,14 @@ bool ProcessesLocal::supportsIoNiceness() {
 long long ProcessesLocal::totalPhysicalMemory() {
     //Try to get the memory via sysconf.  Note the cast to long long to try to avoid a long overflow
     //Should we use sysconf(_SC_PAGESIZE)  or getpagesize()  ?
-    long long memory = ((long long)sysconf(_SC_PHYS_PAGES)) * (sysconf(_SC_PAGESIZE)/1024);
-    if(memory > 0) return memory;
-
-    //This is backup code incase the above failed.  It should never fail on a linux system.
+#ifdef _SC_PHYS_PAGES
+    return ((long long)sysconf(_SC_PHYS_PAGES)) * (sysconf(_SC_PAGESIZE)/1024);
+#else
+    //This is backup code in case this is not defined.  It should never fail on a linux system.
 
     d->mFile.setFileName("/proc/meminfo");
-    if(!d->mFile.open(QIODevice::ReadOnly | QIODevice::Text))
-        return 0; 
+    if(!d->mFile.open(QIODevice::ReadOnly))
+        return 0;
 
     int size;
     while( (size = d->mFile.readLine( d->mBuffer, sizeof(d->mBuffer))) > 0) {  //-1 indicates an error
@@ -515,10 +570,11 @@ long long ProcessesLocal::totalPhysicalMemory() {
 	}
     }
     return 0; // Not found.  Probably will never happen
+#endif
 }
 ProcessesLocal::~ProcessesLocal()
 {
-  delete d;  
+  delete d;
 }
 
 }
